@@ -8,7 +8,6 @@ from collections.abc import Awaitable, Callable, Iterable, Iterator
 from contextlib import contextmanager
 from datetime import datetime
 from functools import partial
-import importlib.resources
 import json
 import os
 from pathlib import Path
@@ -16,6 +15,7 @@ import typing
 from typing import TYPE_CHECKING, Any, TypeVar
 
 import aiohttp
+import aiohttp.typedefs
 import aiohttp.web
 from aiohttp_rpc import JsonRpcMethod, middlewares as rpc_middlewares
 from aiohttp_rpc.errors import InvalidParams, ServerError
@@ -36,16 +36,10 @@ from instawow.cataloguer import CatalogueEntry
 from instawow.common import Flavour, SourceMetadata, infer_flavour_from_path
 from instawow.config import Config, GlobalConfig, SecretStr, config_converter
 from instawow.github_auth import get_codes, poll_for_access_token
-from instawow.manager import (
-    LocksType,
-    Manager,
-    TraceRequestCtx,
-    contextualise,
-    init_web_client,
-    is_outdated,
-)
+from instawow.http import TraceRequestCtx, init_web_client
+from instawow.manager import LocksType, Manager, contextualise, is_outdated
 from instawow.resolvers import Defn
-from instawow.utils import reveal_folder, run_in_thread as t, uniq
+from instawow.utils import read_resource_as_text, reveal_folder, run_in_thread as t, uniq
 
 from . import frontend
 
@@ -55,9 +49,6 @@ if TYPE_CHECKING:
 _T = TypeVar('_T')
 _P = ParamSpec('_P')
 _ManagerBoundCoroFn: TypeAlias = 'Callable[Concatenate[Manager, _P], Awaitable[_T]]'
-_ManagerQueue: TypeAlias = (
-    'asyncio.Queue[tuple[asyncio.Future[object], str, _ManagerBoundCoroFn[..., object]]]'
-)
 
 
 LOCALHOST = '127.0.0.1'
@@ -78,13 +69,20 @@ class _ConfigError(ServerError):
     message = 'invalid configuration parameters'
 
 
+def _extract_loc_from_note(exc: BaseException):
+    notes = getattr(exc, '__notes__', None)
+    if notes is not None:
+        note = notes[-1]
+    else:
+        note = getattr(exc, '__note__', '')
+
+    *_, field = note.rpartition(' ')
+    if field.isidentifier():
+        return field
+
+
 def _structure_excs(excs: Iterable[BaseException]):
-    return [
-        {'loc': [n], 'msg': str(e)}
-        for e in excs
-        for *_, n in (getattr(e, '__note__', '').rpartition(' '),)
-        if n.isidentifier()
-    ]
+    return [{'loc': [l], 'msg': str(e)} for e in excs for l in (_extract_loc_from_note(e),) if l]
 
 
 @contextmanager
@@ -100,17 +98,23 @@ def _reraise_validation_error(
 
 
 @t
-def _read_config(profile: str) -> Config:
+def _read_global_config() -> GlobalConfig:
     with _reraise_validation_error(_ConfigError):
-        return Config.read(GlobalConfig.read(), profile)
+        return GlobalConfig.read()
 
 
-methods: list[tuple[str, type[BaseParams]]] = []
+@t
+def _read_config(global_config: GlobalConfig, profile: str) -> Config:
+    with _reraise_validation_error(_ConfigError):
+        return Config.read(global_config, profile)
+
+
+_methods: list[tuple[str, type[BaseParams]]] = []
 
 
 def _register_method(method: str):
     def inner(param_class: type[BaseParams]):
-        methods.append((method, param_class))
+        _methods.append((method, param_class))
         return frozen(slots=False)(param_class)
 
     return inner
@@ -173,7 +177,7 @@ class WriteProfileConfigParams(_ProfileParamMixin, BaseParams):
 
             # Unload the corresponding ``Manager`` instance for the
             # config to be reloaded on the next request
-            managers.unload(config.profile)
+            managers.unload_profile(config.profile)
 
             return config
 
@@ -183,7 +187,7 @@ class ReadProfileConfigParams(_ProfileParamMixin, BaseParams):
     async def respond(
         self, managers: _ManagerWorkQueue, app_window: toga.MainWindow | None
     ) -> Config:
-        return await _read_config(self.profile)
+        return await _read_config(managers.global_config, self.profile)
 
 
 @_register_method('config/delete_profile')
@@ -193,7 +197,7 @@ class DeleteProfileConfigParams(_ProfileParamMixin, BaseParams):
     ) -> None:
         async def delete_profile(manager: Manager):
             await t(manager.config.delete)()
-            managers.unload(self.profile)
+            managers.unload_profile(self.profile)
 
         await managers.run(self.profile, delete_profile)
 
@@ -208,27 +212,21 @@ class ListProfilesParams(BaseParams):
 
 @_register_method('config/update_global')
 class UpdateGlobalConfigParams(BaseParams):
-    cfcore_access_token: typing.Optional[str]
+    access_tokens: typing.Dict[str, typing.Optional[str]]
 
     async def respond(
         self, managers: _ManagerWorkQueue, app_window: toga.MainWindow | None
     ) -> GlobalConfig:
-        async with managers.locks['update global config']:
-            with _reraise_validation_error(_ConfigError):
-                global_config = await t(GlobalConfig.read)()
-                global_config = evolve(
-                    global_config,
-                    access_tokens=evolve(
-                        global_config.access_tokens,
-                        cfcore=self.cfcore_access_token
-                        if self.cfcore_access_token is None
-                        else SecretStr(self.cfcore_access_token),
-                    ),
-                )
-                await t(global_config.write)()
+        def update_global_config_cb(global_config: GlobalConfig):
+            return evolve(
+                global_config,
+                access_tokens=evolve(
+                    global_config.access_tokens,
+                    **{k: t if t is None else SecretStr(t) for k, t in self.access_tokens.items()},
+                ),
+            )
 
-        managers.unload_all()
-        return global_config
+        return await managers.update_global_config(update_global_config_cb)
 
 
 @_register_method('config/read_global')
@@ -331,12 +329,12 @@ class ErrorResult(TypedDict):
 
 @_register_method('resolve')
 class ResolveParams(_ProfileParamMixin, _DefnParamMixin, BaseParams):
-    async def _resolve(self, manager: Manager) -> dict[Defn, Any]:
+    async def _resolve(self, manager: Manager):
         def extract_source(defn: Defn):
             if defn.source == '*':
-                pair = manager.pair_uri(defn.alias)
-                if pair:
-                    source, alias = pair
+                match = manager.pair_uri(defn.alias)
+                if match:
+                    source, alias = match
                     defn = evolve(defn, source=source, alias=alias)
             return defn
 
@@ -383,6 +381,8 @@ class UpdateParams(_ProfileParamMixin, _DefnParamMixin, BaseParams):
         return [
             {'status': r.status, 'addon': r.new_pkg}
             if isinstance(r, R.PkgUpdated)
+            else {'status': r.status, 'addon': r.pkg}
+            if isinstance(r, R.PkgInstalled)
             else {'status': r.status, 'message': r.message}
             for r in results.values()
         ]
@@ -446,11 +446,14 @@ class AddonMatch_AddonFolder(TypedDict):
 
 @_register_method('reconcile')
 class ReconcileParams(_ProfileParamMixin, BaseParams):
-    matcher: Literal['toc_source_ids', 'folder_name_subsets', 'addon_names_with_folder_names']
+    matcher: Literal[
+        'toc_source_ids', 'folder_hashes', 'folder_name_subsets', 'addon_names_with_folder_names'
+    ]
 
     _MATCHERS = {
         'toc_source_ids': matchers.match_toc_source_ids,
         'folder_name_subsets': matchers.match_folder_name_subsets,
+        'folder_hashes': matchers.match_folder_hashes,
         'addon_names_with_folder_names': matchers.match_addon_names_with_folder_names,
     }
 
@@ -495,14 +498,14 @@ class GetReconcileInstalledCandidatesParams(_ProfileParamMixin, BaseParams):
         self, managers: _ManagerWorkQueue, app_window: toga.MainWindow | None
     ) -> list[ReconcileInstalledCandidate]:
         manager = await managers.get_manager(self.profile)
-        installed_pkgs = (
+        installed_pkgs = [
             models.Pkg.from_row_mapping(manager.database, p)
             for p in manager.database.execute(
                 sa.select(db.pkg).order_by(sa.func.lower(db.pkg.c.name))
             )
             .mappings()
             .all()
-        )
+        ]
         defn_groups = await managers.run(
             self.profile, partial(Manager.find_equivalent_pkg_defns, pkgs=installed_pkgs)
         )
@@ -545,7 +548,7 @@ class GetVersionParams(BaseParams):
     async def respond(
         self, managers: _ManagerWorkQueue, app_window: toga.MainWindow | None
     ) -> GetVersionResult:
-        outdated, new_version = await is_outdated()
+        outdated, new_version = await is_outdated(managers.global_config)
         return {
             'installed_version': __version__,
             'new_version': new_version if outdated else None,
@@ -613,7 +616,7 @@ class ConfirmDialogueParams(BaseParams):
         return {'ok': ok}
 
 
-def _init_json_rpc_web_client():
+def _init_json_rpc_web_client(cache_dir: Path):
     async def do_on_request_end(
         client_session: aiohttp.ClientSession,
         trace_config_ctx: Any,
@@ -629,69 +632,87 @@ def _init_json_rpc_web_client():
             content = params.response.content
             content_length = params.response.content_length
             entry = (
-                trace_request_ctx['manager'],
+                trace_request_ctx['profile'],
                 trace_request_ctx['pkg'],
                 lambda: content.total_bytes / content_length,
             )
             progress_reporters.add(entry)
             content.on_eof(lambda: progress_reporters.remove(entry))
 
-    progress_reporters: set[tuple[Manager, models.Pkg, Callable[[], float]]] = set()
+    progress_reporters: set[tuple[str, models.Pkg, Callable[[], float]]] = set()
 
     trace_config = aiohttp.TraceConfig()
     trace_config.on_request_end.append(do_on_request_end)
     trace_config.freeze()
 
-    return (init_web_client(trace_configs=[trace_config]), progress_reporters)
+    return (init_web_client(cache_dir, trace_configs=[trace_config]), progress_reporters)
 
 
 class _ManagerWorkQueue:
     def __init__(self):
         self._loop = asyncio.get_running_loop()
 
-        self._queue: _ManagerQueue = asyncio.Queue()
+        self._queue: asyncio.Queue[
+            tuple[asyncio.Future[object], str, _ManagerBoundCoroFn[..., object]]
+        ] = asyncio.Queue()
 
         self.locks: LocksType = defaultdict(asyncio.Lock)
 
         self._managers: dict[str, tuple[Manager, Callable[[], None]]] = {}
 
-        self._web_client, self._download_progress_reporters = _init_json_rpc_web_client()
-
         self._github_auth_device_codes = None
         self._github_auth_flow_task = None
 
     async def __aenter__(self):
+        self.global_config = await _read_global_config()
+        self._web_client, self._download_progress_reporters = _init_json_rpc_web_client(
+            self.global_config.cache_dir
+        )
+        contextualise(web_client=self._web_client, locks=self.locks)
         self._listener = self._loop.create_task(self._listen())
 
     async def __aexit__(self, *args: object):
+        self.cancel_github_auth_polling()
         self._listener.cancel()
-        self.unload_all()
+        self._unload_all_profiles()
         await self._web_client.close()
 
-    def unload(self, profile: str):
+    def unload_profile(self, profile: str):
         if profile in self._managers:
-            logger.debug(f'unloading {profile}')
             _, close_db_conn = self._managers.pop(profile)
             close_db_conn()
 
-    def unload_all(self):
+    def _unload_all_profiles(self):
         for profile in list(self._managers):
-            self.unload(profile)
+            self.unload_profile(profile)
 
-    async def _schedule_item(
+    async def update_global_config(
+        self, update_cb: Callable[[GlobalConfig], GlobalConfig]
+    ) -> GlobalConfig:
+        async with self.locks['update global config']:
+            with _reraise_validation_error(_ConfigError):
+                self.global_config = update_cb(self.global_config)
+                await t(self.global_config.write)()
+            self._unload_all_profiles()
+            return self.global_config
+
+    async def _taskify(
         self,
-        future: asyncio.Future[object],
+        future: asyncio.Future[_T],
         profile: str,
-        coro_fn: _ManagerBoundCoroFn[..., object],
+        coro_fn: _ManagerBoundCoroFn[..., _T],
     ):
         try:
-            async with self.locks['modify profile', profile]:
-                try:
-                    manager, _ = self._managers[profile]
-                except KeyError:
-                    config = await _read_config(profile)
-                    self._managers[profile] = Manager.from_config(config)
-                    manager, _ = self._managers[profile]
+            try:
+                manager, _ = self._managers[profile]
+            except KeyError:
+                async with self.locks['modify profile', profile]:
+                    try:
+                        manager, _ = self._managers[profile]
+                    except KeyError:
+                        manager, _ = self._managers[profile] = Manager.from_config(
+                            await _read_config(self.global_config, profile)
+                        )
 
             result = await coro_fn(manager)
         except BaseException as exc:
@@ -700,10 +721,9 @@ class _ManagerWorkQueue:
             future.set_result(result)
 
     async def _listen(self):
-        contextualise(web_client=self._web_client, locks=self.locks)
         while True:
             item = await self._queue.get()
-            asyncio.create_task(self._schedule_item(*item))
+            asyncio.create_task(self._taskify(*item))
             self._queue.task_done()
 
     async def run(self, profile: str, coro_fn: _ManagerBoundCoroFn[..., _T]) -> _T:
@@ -719,7 +739,11 @@ class _ManagerWorkQueue:
 
     async def get_manager_download_progress(self, profile: str):
         manager = await self.get_manager(profile)
-        return ((p, r()) for m, p, r in self._download_progress_reporters if m is manager)
+        return (
+            (p, r())
+            for m, p, r in self._download_progress_reporters
+            if m == manager.config.profile
+        )
 
     async def initiate_github_auth_flow(self):
         async with self.locks['initiate github auth flow']:
@@ -730,17 +754,15 @@ class _ManagerWorkQueue:
                         self._web_client, codes['device_code'], codes['interval']
                     )
 
-                    async with self.locks['update global config']:
-                        global_config = await t(GlobalConfig.read)()
-                        global_config = evolve(
+                    def update_global_config_cb(global_config: GlobalConfig):
+                        return evolve(
                             global_config,
                             access_tokens=evolve(
-                                global_config.access_tokens, cfcore=SecretStr(result)
+                                global_config.access_tokens, github=SecretStr(result)
                             ),
                         )
-                        await t(global_config.write)()
 
-                    self.unload_all()
+                    await self.update_global_config(update_global_config_cb)
 
                 def on_task_complete(future: object):
                     self._github_auth_flow_task = None
@@ -775,7 +797,7 @@ async def create_app(app_window: toga.MainWindow | None = None):
     async def get_index(request: aiohttp.web.Request):
         return aiohttp.web.Response(
             content_type='text/html',
-            text=importlib.resources.read_text(frontend, 'index.html'),
+            text=read_resource_as_text(frontend, 'index.html'),
         )
 
     async def get_static_file(request: aiohttp.web.Request):
@@ -789,7 +811,7 @@ async def create_app(app_window: toga.MainWindow | None = None):
 
         return aiohttp.web.Response(
             content_type=content_type,
-            text=importlib.resources.read_text(frontend, filename),
+            text=read_resource_as_text(frontend, filename),
         )
 
     def json_serialize(value: dict[str, Any]):
@@ -799,12 +821,12 @@ async def create_app(app_window: toga.MainWindow | None = None):
         json_serialize=json_serialize,
         middlewares=rpc_middlewares.DEFAULT_MIDDLEWARES,
     )
-    rpc_server.add_methods([m.bind(n, managers, app_window) for n, m in methods])
+    rpc_server.add_methods([m.bind(n, managers, app_window) for n, m in _methods])
 
     @aiohttp.web.middleware
     async def enforce_same_origin(
         request: aiohttp.web.Request,
-        handler: Callable[[aiohttp.web.Request], Awaitable[aiohttp.web.StreamResponse]],
+        handler: aiohttp.typedefs.Handler,
     ):
         if request.remote != LOCALHOST:
             raise aiohttp.web.HTTPUnauthorized
