@@ -2,6 +2,6 @@
 
 import typing as _typing
 
-if _typing.TYPE_CHECKING:
+if _typing.TYPE_CHECKING:  # pragma: no cover
     import aiohttp
     import prompt_toolkit.shortcuts
