@@ -6,11 +6,15 @@ import typing
 from attrs import asdict, frozen
 from cattrs import GenConverter
 from cattrs.preconf.json import configure_converter
+from typing_extensions import Self
 
-from . import manager
 from .config import Flavour
 from .utils import bucketise, cached_property, normalise_names
 
+BASE_CATALOGUE_VERSION = 7
+CATALOGUE_VERSION = 4
+
+
 catalogue_converter = GenConverter(
     unstruct_collection_overrides={
         # TODO: Replace with ``collections.abc.Set``
@@ -48,13 +52,16 @@ class CatalogueEntry(BaseCatalogueEntry):
 
 @frozen(kw_only=True)
 class BaseCatalogue:
-    version: int = 5
+    version: int = BASE_CATALOGUE_VERSION
     entries: typing.List[BaseCatalogueEntry]
 
     @classmethod
-    async def collate(cls, start_date: datetime | None):
-        async with manager.init_web_client() as web_client:
-            entries = [e for r in manager.Manager.RESOLVERS async for e in r.catalogue(web_client)]
+    async def collate(cls, start_date: datetime | None) -> Self:
+        from .http import init_web_client
+        from .manager import Manager
+
+        async with init_web_client(None) as web_client:
+            entries = [e for r in Manager.RESOLVERS async for e in r.catalogue(web_client)]
             if start_date is not None:
                 entries = [e for e in entries if e.last_updated >= start_date]
             return cls(entries=entries)
@@ -62,13 +69,16 @@ class BaseCatalogue:
 
 @frozen(kw_only=True, slots=False)
 class Catalogue:
-    version: int = 3
+    version: int = CATALOGUE_VERSION
     entries: typing.List[CatalogueEntry]
     curse_slugs: typing.Dict[str, str]
 
     @classmethod
-    def from_base_catalogue(cls, unstructured_base_catalogue: object, start_date: datetime | None):
-        from .resolvers import CfCoreResolver, GithubResolver
+    def from_base_catalogue(
+        cls, unstructured_base_catalogue: object, start_date: datetime | None
+    ) -> Self:
+        from ._sources.cfcore import CfCoreResolver
+        from ._sources.github import GithubResolver
 
         normalise = normalise_names('')
 
@@ -93,16 +103,14 @@ class Catalogue:
         }
         entries = [
             CatalogueEntry(
-                **{
-                    **asdict(e),
-                    'same_as': e.same_as
-                    if e.source == GithubResolver.metadata.id
-                    else (same_as_from_github.get((e.source, e.id)) or e.same_as),
-                    'normalised_name': normalise(e.name),
-                    'derived_download_score': 0
-                    if e.source == GithubResolver.metadata.id
-                    else e.download_count / most_downloads_per_source[e.source],
-                }
+                **asdict(e, filter=lambda a, _: a.name != 'same_as'),
+                same_as=e.same_as
+                if e.source == GithubResolver.metadata.id
+                else (same_as_from_github.get((e.source, e.id)) or e.same_as),
+                normalised_name=normalise(e.name),
+                derived_download_score=0
+                if e.source == GithubResolver.metadata.id
+                else e.download_count / most_downloads_per_source[e.source],
             )
             for e in base_entries
         ]
