@@ -1,16 +1,7 @@
 from __future__ import annotations
 
 import asyncio
-from collections.abc import (
-    Awaitable,
-    Callable,
-    Collection,
-    Generator,
-    Iterable,
-    Iterator,
-    Sequence,
-    Set,
-)
+from collections.abc import Awaitable, Callable, Collection, Iterable, Iterator, Sequence, Set
 from contextlib import contextmanager
 from datetime import datetime, timezone
 from functools import partial
@@ -24,14 +15,14 @@ import click
 from loguru import logger
 
 from . import __version__, _deferred_types, db, manager as _manager, models, results as R
-from .common import Flavour, Strategy
+from .common import ChangelogFormat, Flavour, Strategy
 from .config import Config, GlobalConfig, config_converter, setup_logging
+from .http import TraceRequestCtx, init_web_client
 from .plugins import load_plugins
-from .resolvers import ChangelogFormat, Defn
+from .resolvers import Defn
 from .utils import StrEnum, all_eq, cached_property, gather, reveal_folder, tabulate, uniq
 
 _T = TypeVar('_T')
-_F = TypeVar('_F', bound='Callable[..., object]')
 
 
 class Report:
@@ -65,7 +56,7 @@ class Report:
 
     def __str__(self) -> str:
         return '\n'.join(
-            f'{self._result_type_to_symbol(r)} {click.style(a.to_urn(), bold=True)}\n'
+            f'{self._result_type_to_symbol(r)} {click.style(defn_to_urn(a), bold=True)}\n'
             f'{textwrap.fill(r.message, initial_indent=" " * 2, subsequent_indent=" " * 4)}'
             for a, r in self.results
             if self.filter_fn(r)
@@ -73,8 +64,10 @@ class Report:
 
     def generate(self) -> None:
         mw: _CtxObjWrapper | None = click.get_current_context().obj
-        if mw and mw.manager.config.global_config.auto_update_check:
-            outdated, new_version = mw.run_with_progress(_manager.is_outdated())
+        if mw:
+            outdated, new_version = mw.run_with_progress(
+                _manager.is_outdated(mw.manager.config.global_config)
+            )
             if outdated:
                 click.echo(f'{self.WARNING_SYMBOL} instawow-{new_version} is available')
 
@@ -89,6 +82,7 @@ class Report:
 
 
 def _init_cli_web_client(
+    cache_dir: Path,
     progress_bar: _deferred_types.prompt_toolkit.shortcuts.ProgressBar,
     tickers: set[asyncio.Task[None]],
 ) -> _deferred_types.aiohttp.ClientSession:
@@ -102,12 +96,12 @@ def _init_cli_web_client(
         trace_config_ctx: Any,
         params: _deferred_types.aiohttp.TraceRequestEndParams,
     ):
-        trace_request_ctx: _manager.TraceRequestCtx = trace_config_ctx.trace_request_ctx
+        trace_request_ctx: TraceRequestCtx = trace_config_ctx.trace_request_ctx
         if trace_request_ctx:
             response = params.response
             label = (
                 'Downloading '
-                + Defn(trace_request_ctx['pkg'].source, trace_request_ctx['pkg'].slug).to_urn()
+                + defn_to_urn(Defn(trace_request_ctx['pkg'].source, trace_request_ctx['pkg'].slug))
                 if trace_request_ctx['report_progress'] == 'pkg_download'
                 else trace_request_ctx['label']
             )
@@ -123,7 +117,7 @@ def _init_cli_web_client(
                 progress_bar.counters.append(counter)
                 try:
                     while not response.content.is_eof():
-                        counter.items_completed = response.content.total_bytes  # type: ignore
+                        counter.items_completed = response.content.total_bytes
                         progress_bar.invalidate()
                         await asyncio.sleep(TICK_INTERVAL)
                 finally:
@@ -134,7 +128,7 @@ def _init_cli_web_client(
     trace_config = TraceConfig()
     trace_config.on_request_end.append(do_on_request_end)
     trace_config.freeze()
-    return _manager.init_web_client(trace_configs=[trace_config])
+    return init_web_client(cache_dir, trace_configs=[trace_config])
 
 
 @contextmanager
@@ -173,26 +167,31 @@ class _CtxObjWrapper:
         except FileNotFoundError:
             config = self._ctx.invoke(configure)
 
-        setup_logging(config.logging_dir, self._ctx.params['debug'])
+        setup_logging(config.logging_dir, *self._ctx.params['debug'])
+
         manager, close_db_conn = _manager.Manager.from_config(config)
         self._ctx.call_on_close(close_db_conn)
 
         return manager
 
     def run_with_progress(self, awaitable: Awaitable[_T]) -> _T:
-        if self._ctx.params['debug']:
+        cache_dir = self.manager.config.global_config.cache_dir
+
+        if any(self._ctx.params['debug']):
 
             async def run():
-                async with _manager.init_web_client() as web_client:
+                async with init_web_client(cache_dir) as web_client:
                     _manager.contextualise(web_client=web_client)
                     return await awaitable
 
         else:
-            from .prompts import make_progress_bar
+            from ._cli_prompts import make_progress_bar
 
             async def run():
                 with make_progress_bar() as progress_bar, _cancel_tickers(progress_bar) as tickers:
-                    async with _init_cli_web_client(progress_bar, tickers) as web_client:
+                    async with _init_cli_web_client(
+                        cache_dir, progress_bar, tickers
+                    ) as web_client:
                         _manager.contextualise(web_client=web_client)
                         return await awaitable
 
@@ -213,7 +212,7 @@ class StrEnumParam(click.Choice):
         excludes: Set[StrEnum] = frozenset(),
         case_sensitive: bool = True,
     ) -> None:
-        self.choice_enum = choice_enum
+        self._choice_enum = choice_enum
         super().__init__(
             choices=[c for c in choice_enum if c not in excludes],
             case_sensitive=case_sensitive,
@@ -222,7 +221,7 @@ class StrEnumParam(click.Choice):
     def convert(
         self, value: str, param: click.Parameter | None, ctx: click.Context | None
     ) -> StrEnum:
-        return self.choice_enum(super().convert(value, param, ctx))
+        return self._choice_enum(super().convert(value, param, ctx))
 
 
 def _register_plugin_commands(group: click.Group):
@@ -233,17 +232,25 @@ def _register_plugin_commands(group: click.Group):
     return group
 
 
+def defn_to_urn(defn: Defn) -> str:
+    return f'{defn.source}:{defn.alias}'
+
+
+def _parse_debug_option(
+    _: click.Context, __: click.Parameter, value: float
+) -> tuple[bool, bool, bool]:
+    return (value > 0, value > 1, value > 2)
+
+
 @_register_plugin_commands
-@click.group(  # pyright: ignore[reportUnknownMemberType]
-    context_settings={'help_option_names': ('-h', '--help')}
-)
+@click.group(context_settings={'help_option_names': ('-h', '--help')})
 @click.version_option(__version__, prog_name=__package__)
 @click.option(
     '--debug',
-    'debug',
-    is_flag=True,
-    default=False,
-    help='Log more things.',
+    '-d',
+    count=True,
+    help='Log incrementally more things.  Additive.',
+    callback=_parse_debug_option,
 )
 @click.option(
     '--profile',
@@ -262,30 +269,32 @@ main = logger.catch(reraise=True)(cli)
 
 
 @overload
-def parse_into_defn(manager: _manager.Manager, value: str, *, raise_invalid: bool = True) -> Defn:
+def _parse_into_defn(manager: _manager.Manager, value: str, *, raise_invalid: bool = True) -> Defn:
     ...
 
 
 @overload
-def parse_into_defn(
+def _parse_into_defn(
     manager: _manager.Manager, value: list[str], *, raise_invalid: bool = True
 ) -> list[Defn]:
     ...
 
 
 @overload
-def parse_into_defn(manager: _manager.Manager, value: None, *, raise_invalid: bool = True) -> None:
+def _parse_into_defn(
+    manager: _manager.Manager, value: None, *, raise_invalid: bool = True
+) -> None:
     ...
 
 
-def parse_into_defn(
+def _parse_into_defn(
     manager: _manager.Manager, value: str | list[str] | None, *, raise_invalid: bool = True
 ) -> Defn | list[Defn] | None:
     if value is None:
         return None
 
     if not isinstance(value, str):
-        defns = (parse_into_defn(manager, v, raise_invalid=raise_invalid) for v in value)
+        defns = (_parse_into_defn(manager, v, raise_invalid=raise_invalid) for v in value)
         return uniq(defns)
 
     pair = manager.pair_uri(value)
@@ -297,17 +306,17 @@ def parse_into_defn(
     return Defn(*pair)
 
 
-def parse_into_defn_with_strategy(
+def _parse_into_defn_with_strategy(
     manager: _manager.Manager, value: Sequence[tuple[Strategy, str]]
 ) -> Iterator[Defn]:
-    defns = parse_into_defn(manager, [d for _, d in value])
+    defns = _parse_into_defn(manager, [d for _, d in value])
     return map(lambda d, s: evolve(d, strategy=s), defns, (s for s, _ in value))
 
 
-def parse_into_defn_with_version(
+def _parse_into_defn_with_version(
     manager: _manager.Manager, value: Sequence[tuple[str, str]]
 ) -> Iterator[Defn]:
-    defns = parse_into_defn(manager, [d for _, d in value])
+    defns = _parse_into_defn(manager, [d for _, d in value])
     return map(Defn.with_version, defns, (v for v, _ in value))
 
 
@@ -327,7 +336,7 @@ _EXCLUDED_STRATEGIES = frozenset({Strategy.default, Strategy.version})
 
 @cli.command()
 @click.argument(
-    'addons', nargs=-1, callback=partial(_combine_addons, parse_into_defn), expose_value=False
+    'addons', nargs=-1, callback=partial(_combine_addons, _parse_into_defn), expose_value=False
 )
 @click.option(
     '--with-strategy',
@@ -335,17 +344,19 @@ _EXCLUDED_STRATEGIES = frozenset({Strategy.default, Strategy.version})
     multiple=True,
     type=(StrEnumParam(Strategy, _EXCLUDED_STRATEGIES), str),
     expose_value=False,
-    callback=partial(_combine_addons, parse_into_defn_with_strategy),
+    callback=partial(_combine_addons, _parse_into_defn_with_strategy),
     metavar='<STRATEGY ADDON>...',
-    help='A strategy followed by an add-on definition.  '
-    f'One of: {", ".join(s for s in Strategy if s not in _EXCLUDED_STRATEGIES)}.',
+    help=(
+        'A strategy followed by an add-on definition.  '
+        f'One of: {", ".join(s for s in Strategy if s not in _EXCLUDED_STRATEGIES)}.'
+    ),
 )
 @click.option(
     '--version',
     multiple=True,
     type=(str, str),
     expose_value=False,
-    callback=partial(_combine_addons, parse_into_defn_with_version),
+    callback=partial(_combine_addons, _parse_into_defn_with_version),
     metavar='<VERSION ADDON>...',
     help='A version followed by an add-on definition.',
 )
@@ -363,7 +374,7 @@ def install(mw: _CtxObjWrapper, addons: Sequence[Defn], replace: bool) -> None:
 
 
 @cli.command()
-@click.argument('addons', nargs=-1, callback=_with_manager(parse_into_defn))
+@click.argument('addons', nargs=-1, callback=_with_manager(_parse_into_defn))
 @click.pass_obj
 def update(mw: _CtxObjWrapper, addons: Sequence[Defn]) -> None:
     "Update installed add-ons."
@@ -397,7 +408,7 @@ def update(mw: _CtxObjWrapper, addons: Sequence[Defn]) -> None:
 
 
 @cli.command()
-@click.argument('addons', nargs=-1, required=True, callback=_with_manager(parse_into_defn))
+@click.argument('addons', nargs=-1, required=True, callback=_with_manager(_parse_into_defn))
 @click.option(
     '--keep-folders',
     is_flag=True,
@@ -412,7 +423,7 @@ def remove(mw: _CtxObjWrapper, addons: Sequence[Defn], keep_folders: bool) -> No
 
 
 @cli.command()
-@click.argument('addon', callback=_with_manager(parse_into_defn))
+@click.argument('addon', callback=_with_manager(_parse_into_defn))
 @click.option(
     '--version',
     help='Version to roll back to.',
@@ -426,7 +437,7 @@ def remove(mw: _CtxObjWrapper, addons: Sequence[Defn], keep_folders: bool) -> No
 @click.pass_obj
 def rollback(mw: _CtxObjWrapper, addon: Defn, version: str | None, undo: bool) -> None:
     "Roll an add-on back to an older version."
-    from .prompts import Choice, ask, select
+    from ._cli_prompts import Choice, ask, select
 
     if version and undo:
         raise click.UsageError('Cannot use "--version" with "--undo"')
@@ -461,7 +472,7 @@ def rollback(mw: _CtxObjWrapper, addon: Defn, version: str | None, undo: bool) -
             for v in versions
         ]
         selection = ask(
-            select(f'Select version of {reconstructed_defn.to_urn()} for rollback', choices)
+            select(f'Select version of {defn_to_urn(reconstructed_defn)} for rollback', choices)
         )
 
     Report(
@@ -486,20 +497,21 @@ def rollback(mw: _CtxObjWrapper, addon: Defn, version: str | None, undo: bool) -
 @click.pass_obj
 def reconcile(mw: _CtxObjWrapper, auto: bool, rereconcile: bool, list_unreconciled: bool) -> None:
     "Reconcile pre-installed add-ons."
+    from ._cli_prompts import PkgChoice, ask, confirm, select, skip
     from .matchers import (
         AddonFolder,
         get_unreconciled_folders,
         match_addon_names_with_folder_names,
+        match_folder_hashes,
         match_folder_name_subsets,
         match_toc_source_ids,
     )
-    from .prompts import PkgChoice, ask, confirm, select, skip
 
     def construct_choice(pkg: models.Pkg, highlight_version: bool, disabled: bool):
         defn = Defn.from_pkg(pkg)
         return PkgChoice(
             [
-                ('', f'{defn.to_urn()}=='),
+                ('', f'{defn_to_urn(defn)}=='),
                 ('class:highlight-sub' if highlight_version else '', pkg.version),
             ],
             pkg=pkg,
@@ -508,8 +520,8 @@ def reconcile(mw: _CtxObjWrapper, auto: bool, rereconcile: bool, list_unreconcil
         )
 
     def gather_selections(
-        groups: Sequence[tuple[Any, Sequence[Defn]]],
-        selector: Callable[[Any, Sequence[models.Pkg]], Defn | None],
+        groups: Collection[tuple[_T, Sequence[Defn]]],
+        selector: Callable[[_T, Sequence[models.Pkg]], Defn | None],
     ):
         results = mw.run_with_progress(mw.manager.resolve(uniq(d for _, b in groups for d in b)))
         for addons_or_pkg, defns in groups:
@@ -526,7 +538,7 @@ def reconcile(mw: _CtxObjWrapper, auto: bool, rereconcile: bool, list_unreconcil
 
         import sqlalchemy as sa
 
-        def prompt_reconciled(installed_pkg: models.Pkg, pkgs: Sequence[models.Pkg]):
+        def select_alternative_pkg(installed_pkg: models.Pkg, pkgs: Sequence[models.Pkg]):
             highlight_version = not all_eq(i.version for i in (installed_pkg, *pkgs))
             choices = [
                 construct_choice(installed_pkg, highlight_version, True),
@@ -536,15 +548,21 @@ def reconcile(mw: _CtxObjWrapper, auto: bool, rereconcile: bool, list_unreconcil
             selection = ask(select(installed_pkg.name, choices))
             return selection or None
 
-        installed_pkgs = (
+        installed_pkgs = [
             models.Pkg.from_row_mapping(mw.manager.database, p)
-            for p in mw.manager.database.execute(sa.select(db.pkg)).mappings().all()
-        )
-        groups = list(
-            mw.run_with_progress(mw.manager.find_equivalent_pkg_defns(installed_pkgs)).items()
-        )
+            for p in mw.manager.database.execute(
+                sa.select(db.pkg).order_by(sa.func.lower(db.pkg.c.name))
+            )
+            .mappings()
+            .all()
+        ]
+        groups = mw.run_with_progress(mw.manager.find_equivalent_pkg_defns(installed_pkgs))
         selections = [
-            (p, s) for (p, _), s in zip(groups, gather_selections(groups, prompt_reconciled)) if s
+            (p, s)
+            for (p, _), s in zip(
+                groups.items(), gather_selections(groups.items(), select_alternative_pkg)
+            )
+            if s
         ]
         if selections and ask(confirm('Install selected add-ons?')):
             Report(
@@ -558,80 +576,80 @@ def reconcile(mw: _CtxObjWrapper, auto: bool, rereconcile: bool, list_unreconcil
                 ).items()
             ).generate_and_exit()
 
-        return
+    else:
+        PREAMBLE = textwrap.dedent(
+            '''\
+            Use the arrow keys to navigate, <o> to open an add-on in your browser,
+            enter to make a selection and <s> to skip to the next item.
 
-    preamble = textwrap.dedent(
-        '''\
-        Use the arrow keys to navigate, <o> to open an add-on in your browser,
-        enter to make a selection and <s> to skip to the next item.
+            Versions that differ from the installed version or differ between
+            choices are highlighted in purple.
 
-        Versions that differ from the installed version or differ between
-        choices are highlighted in purple.
+            instawow will perform three passes in decreasing order of accuracy,
+            looking to match source IDs and add-on names in TOC files, and folders.
 
-        The reconciler will perform three passes in decreasing order of accuracy,
-        looking to match source IDs and add-on names in TOC files, and folders.
+            Selected add-ons will be reinstalled.
 
-        Selected add-ons will be reinstalled.
+            You can also run `reconcile` in automatic mode by passing
+            the `--auto` flag.  In this mode, add-ons will be reconciled
+            without user input.
+            '''
+        )
 
-        You can also run `reconcile` in promptless mode by passing
-        the `--auto` flag.  In this mode, add-ons will be reconciled
-        without user input.
-        '''
-    )
+        leftovers = get_unreconciled_folders(mw.manager)
+        if list_unreconciled:
+            table_rows = [('unreconciled',), *((f.name,) for f in sorted(leftovers))]
+            click.echo(tabulate(table_rows))
+            return
+        elif not leftovers:
+            click.echo('No add-ons left to reconcile.')
+            return
 
-    def prompt_unreconciled(addons: Sequence[AddonFolder], pkgs: Sequence[models.Pkg]):
-        def combine_names():
-            return textwrap.shorten(', '.join(a.name for a in addons), 60)
+        if not auto:
+            click.echo(PREAMBLE)
 
-        # Highlight version if there's multiple of them
-        highlight_version = not all_eq(i.version for i in chain(addons, pkgs))
-        choices = [
-            *(construct_choice(p, highlight_version, False) for p in pkgs),
-            skip,
-        ]
-        selection = ask(select(f'{combine_names()} [{addons[0].version or "?"}]', choices))
-        return selection or None
+        def select_pkg(addons: Sequence[AddonFolder], pkgs: Sequence[models.Pkg]):
+            def combine_names():
+                return textwrap.shorten(', '.join(a.name for a in addons), 60)
 
-    def pick_first(addons: Sequence[AddonFolder], pkgs: Sequence[models.Pkg]):
-        return Defn.from_pkg(pkgs[0])
+            # Highlight version if there's multiple of them
+            highlight_version = not all_eq(i.version for i in chain(addons, pkgs))
+            choices = [
+                *(construct_choice(p, highlight_version, False) for p in pkgs),
+                skip,
+            ]
+            selection = ask(select(f'{combine_names()} [{addons[0].version or "?"}]', choices))
+            return selection or None
+
+        def pick_first_pkg(addons: Sequence[AddonFolder], pkgs: Sequence[models.Pkg]):
+            return Defn.from_pkg(pkgs[0])
+
+        select_pkg_ = pick_first_pkg if auto else select_pkg
+        confirm_install = (
+            (lambda: True) if auto else (lambda: ask(confirm('Install selected add-ons?')))
+        )
 
-    def match_all(
-        selector: Callable[[Any, Sequence[models.Pkg]], Defn | None]
-    ) -> Generator[list[Defn], frozenset[AddonFolder], None]:
         # Match in order of increasing heuristicitivenessitude
         for fn in [
             match_toc_source_ids,
             match_folder_name_subsets,
+            match_folder_hashes,
             match_addon_names_with_folder_names,
         ]:
-            groups = mw.run_with_progress(fn(mw.manager, (yield [])))
-            yield list(filter(None, gather_selections(groups, selector)))
+            groups = mw.run_with_progress(fn(mw.manager, leftovers))
+            selections = [s for s in gather_selections(groups, select_pkg_) if s is not None]
+            if selections and confirm_install():
+                results = mw.run_with_progress(mw.manager.install(selections, True))
+                Report(results.items()).generate()
 
-    leftovers = get_unreconciled_folders(mw.manager)
-    if list_unreconciled:
-        table_rows = [('unreconciled',), *((f.name,) for f in sorted(leftovers))]
-        click.echo(tabulate(table_rows))
-        return
-    elif not leftovers:
-        click.echo('No add-ons left to reconcile.')
-        return
+            leftovers = get_unreconciled_folders(mw.manager)
+            if not leftovers:
+                break
 
-    if not auto:
-        click.echo(preamble)
-
-    matcher = match_all(pick_first if auto else prompt_unreconciled)
-    for _ in matcher:  # Skip over consumer yields
-        selections = matcher.send(leftovers)
-        if selections and (auto or ask(confirm('Install selected add-ons?'))):
-            results = mw.run_with_progress(mw.manager.install(selections, True))
-            Report(results.items()).generate()
-
-        leftovers = get_unreconciled_folders(mw.manager)
-
-    if leftovers:
-        click.echo()
-        table_rows = [('unreconciled',), *((f.name,) for f in sorted(leftovers))]
-        click.echo(tabulate(table_rows))
+        if leftovers:
+            click.echo()
+            table_rows = [('unreconciled',), *((f.name,) for f in sorted(leftovers))]
+            click.echo(tabulate(table_rows))
 
 
 def _concat_search_terms(_: click.Context, __: click.Parameter, value: tuple[str, ...]):
@@ -673,7 +691,7 @@ def search(
     start_date: datetime | None,
 ) -> None:
     "Search for add-ons to install."
-    from .prompts import PkgChoice, ask, checkbox, confirm
+    from ._cli_prompts import PkgChoice, ask, checkbox, confirm
 
     mw: _CtxObjWrapper = ctx.obj
 
@@ -686,7 +704,7 @@ def search(
         for d, r in mw.run_with_progress(mw.manager.resolve(defns)).items()
         if isinstance(r, models.Pkg)
     )
-    choices = [PkgChoice(f'{p.name}  ({d.to_urn()}=={p.version})', d, pkg=p) for d, p in pkgs]
+    choices = [PkgChoice(f'{p.name}  ({defn_to_urn(d)}=={p.version})', d, pkg=p) for d, p in pkgs]
     if choices:
         selections: list[Defn] = ask(checkbox('Select add-ons to install', choices=choices))
         if selections and ask(confirm('Install selected add-ons?')):
@@ -695,7 +713,7 @@ def search(
         click.echo('No results found.')
 
 
-class _ListFormats(StrEnum):
+class _ListFormat(StrEnum):
     simple = 'simple'
     detailed = 'detailed'
     json = 'json'
@@ -703,27 +721,25 @@ class _ListFormats(StrEnum):
 
 @cli.command('list')
 @click.argument(
-    'addons', nargs=-1, callback=_with_manager(partial(parse_into_defn, raise_invalid=False))
+    'addons', nargs=-1, callback=_with_manager(partial(_parse_into_defn, raise_invalid=False))
 )
 @click.option(
     '--format',
     '-f',
     'output_format',
-    type=StrEnumParam(_ListFormats),
-    default=_ListFormats.simple,
+    type=StrEnumParam(_ListFormat),
+    default=_ListFormat.simple,
     show_default=True,
     help='Change the output format.',
 )
 @click.pass_obj
-def list_installed(
-    mw: _CtxObjWrapper, addons: Sequence[Defn], output_format: _ListFormats
-) -> None:
+def list_installed(mw: _CtxObjWrapper, addons: Sequence[Defn], output_format: _ListFormat) -> None:
     "List installed add-ons."
     import sqlalchemy as sa
 
     def format_deps(pkg: models.Pkg):
         return (
-            Defn(pkg.source, s or e.id).to_urn()
+            defn_to_urn(Defn(pkg.source, s or e.id))
             for e in pkg.deps
             for s in (
                 mw.manager.database.execute(
@@ -751,13 +767,13 @@ def list_installed(
                 if addons
                 else True
             )
-            .order_by(db.pkg.c.source, db.pkg.c.name)
+            .order_by(db.pkg.c.source, sa.func.lower(db.pkg.c.name))
         )
         .mappings()
         .all()
     )
 
-    if output_format is _ListFormats.json:
+    if output_format is _ListFormat.json:
         import json
         import typing
 
@@ -770,10 +786,10 @@ def list_installed(
             )
         )
 
-    elif output_format is _ListFormats.detailed:
+    elif output_format is _ListFormat.detailed:
         formatter = click.HelpFormatter(max_width=99)
         for pkg in row_mappings_to_pkgs():
-            with formatter.section(Defn.from_pkg(pkg).to_urn()):
+            with formatter.section(defn_to_urn(Defn.from_pkg(pkg))):
                 formatter.write_dl(
                     [
                         ('name', pkg.name),
@@ -790,21 +806,21 @@ def list_installed(
 
     else:
         click.echo(
-            ''.join(f'{Defn(p["source"], p["slug"]).to_urn()}\n' for p in pkg_mappings),
+            ''.join(f'{defn_to_urn(Defn(p["source"], p["slug"]))}\n' for p in pkg_mappings),
             nl=False,
         )
 
 
 @cli.command(hidden=True)
-@click.argument('addon', callback=_with_manager(partial(parse_into_defn, raise_invalid=False)))
+@click.argument('addon', callback=_with_manager(partial(_parse_into_defn, raise_invalid=False)))
 @click.pass_context
 def info(ctx: click.Context, addon: Defn) -> None:
     "Alias of `list -f detailed`."
-    ctx.invoke(list_installed, addons=(addon,), output_format=_ListFormats.detailed)
+    ctx.invoke(list_installed, addons=(addon,), output_format=_ListFormat.detailed)
 
 
 @cli.command()
-@click.argument('addon', callback=_with_manager(partial(parse_into_defn, raise_invalid=False)))
+@click.argument('addon', callback=_with_manager(partial(_parse_into_defn, raise_invalid=False)))
 @click.pass_obj
 def reveal(mw: _CtxObjWrapper, addon: Defn) -> None:
     "Bring an add-on up in your file manager."
@@ -817,7 +833,7 @@ def reveal(mw: _CtxObjWrapper, addon: Defn) -> None:
 
 @cli.command()
 @click.argument(
-    'addon', callback=_with_manager(partial(parse_into_defn, raise_invalid=False)), required=False
+    'addon', callback=_with_manager(partial(_parse_into_defn, raise_invalid=False)), required=False
 )
 @click.option(
     '--convert/--no-convert',
@@ -833,6 +849,8 @@ def view_changelog(mw: _CtxObjWrapper, addon: Defn | None, convert: bool) -> Non
     to have been installed within one minute of the last add-on.
     """
 
+    MAX_LINES = 100
+
     def make_converter():
         import shutil
 
@@ -858,6 +876,16 @@ def view_changelog(mw: _CtxObjWrapper, addon: Defn | None, convert: bool) -> Non
 
             return real_convert
 
+    def format_combined_changelog_entry(source: str, slug: str, changelog: str):
+        lines = changelog.splitlines()
+        body = '\n'.join(
+            (
+                *(f'  {l}' for l in lines[:MAX_LINES]),
+                *(('  [...]',) if len(lines) > MAX_LINES else ()),
+            )
+        )
+        return f'{defn_to_urn(Defn(source, slug))}:\n{body}'
+
     if addon:
         pkg = mw.manager.get_pkg(addon, partial_match=True)
         if pkg:
@@ -902,7 +930,7 @@ def view_changelog(mw: _CtxObjWrapper, addon: Defn | None, convert: bool) -> Non
             )
         click.echo_via_pager(
             '\n\n'.join(
-                Defn(m.source, m.slug).to_urn() + ':\n' + textwrap.indent(c, '  ')
+                format_combined_changelog_entry(m.source, m.slug, c)
                 for m, c in zip(last_installed_changelog_urls, changelogs)
             )
         )
@@ -917,7 +945,7 @@ def _show_active_config(ctx: click.Context, __: click.Parameter, value: bool):
 async def _github_oauth_flow():
     from .github_auth import get_codes, poll_for_access_token
 
-    async with _manager.init_web_client() as web_client:
+    async with init_web_client(None) as web_client:
         codes = await get_codes(web_client)
         click.echo(f'Navigate to {codes["verification_uri"]} and paste the code below:')
         click.echo(f'  {codes["user_code"]}')
@@ -934,6 +962,7 @@ class _EditableConfigOptions(StrEnum):
     auto_update_check = 'auto_update_check'
     github_access_token = 'access_tokens.github'
     cfcore_access_token = 'access_tokens.cfcore'
+    wago_access_token = 'access_tokens.wago'
 
 
 @cli.command()
@@ -957,24 +986,26 @@ def configure(
     config_options: Collection[_EditableConfigOptions],
 ) -> Config:
     "Configure instawow."
+    from ._cli_prompts import AttrFieldValidator, ask, confirm, password, path, select
     from .common import infer_flavour_from_path
-    from .prompts import AttrFieldValidator, ask, confirm, password, path, select
+
+    profile = ctx.find_root().params['profile']
+
+    orig_global_config = GlobalConfig.read()
 
     if not config_options:
         config_options = {
             _EditableConfigOptions.addon_dir,
             _EditableConfigOptions.game_flavour,
-            _EditableConfigOptions.github_access_token,
         }
+        if orig_global_config.access_tokens.github is None:
+            config_options.add(_EditableConfigOptions.github_access_token)
 
-    profile = ctx.find_root().params['profile']
-
-    orig_global_config = GlobalConfig.read()
     global_config_values = asdict(orig_global_config)
     try:
         profile_config_values = asdict(Config.read(orig_global_config, profile))
         del profile_config_values['global_config']
-    except FileNotFoundError:
+    except Exception:
         profile_config_values = {
             'profile': profile,
         }
@@ -986,7 +1017,8 @@ def configure(
                 'Add-on directory:',
                 only_directories=True,
                 validate=AttrFieldValidator(
-                    fields(resolve_types(Config)).addon_dir, config_converter
+                    fields(resolve_types(Config)).addon_dir,  # pyright: ignore
+                    config_converter,
                 ),
             )
         )
@@ -1016,8 +1048,26 @@ def configure(
         global_config_values['access_tokens']['github'] = github_access_token
 
     if _EditableConfigOptions.cfcore_access_token in config_options:
+        click.echo(
+            textwrap.fill(
+                'An access token is required to use CurseForge. '
+                'Log in to https://console.curseforge.com/ to generate an access token.'
+            )
+        )
         global_config_values['access_tokens']['cfcore'] = ask(
-            password('Enter CFCore access token:')
+            password('CFCore access token:', validate=bool)
+        )
+
+    if _EditableConfigOptions.wago_access_token in config_options:
+        click.echo(
+            textwrap.fill(
+                'An access token is required to use Wago Addons. '
+                'Wago issues tokens to Patreon subscribers above a certain tier. '
+                'See https://addons.wago.io/patreon for more information.'
+            )
+        )
+        global_config_values['access_tokens']['wago_addons'] = ask(
+            password('Wago Addons access token:', validate=bool)
         )
 
     global_config = config_converter.structure(global_config_values, GlobalConfig).write()
@@ -1101,7 +1151,8 @@ def gui(ctx: click.Context) -> None:
     dummy_jsonrpc_config = Config.make_dummy_config(
         global_config=global_config, profile='__jsonrpc__'
     ).ensure_dirs()
+
     params = ctx.find_root().params
-    setup_logging(dummy_jsonrpc_config.logging_dir, params['debug'])
+    setup_logging(dummy_jsonrpc_config.logging_dir, *params['debug'])
 
     InstawowApp(version=__version__).main_loop()  # pyright: ignore[reportUnknownMemberType]
