@@ -1,6 +1,7 @@
 from __future__ import annotations
 
-from collections.abc import Callable, Iterable, Set, Sized
+from collections.abc import Callable, Iterable, Sized
+from functools import lru_cache
 import json
 import os
 from pathlib import Path
@@ -52,21 +53,23 @@ def _enrich_validator_exc(validator: Callable[[object, Attribute[_T], _T], None]
             validator(model, attr, value)
         except BaseException as exc:
             note = f'Structuring class {model.__class__.__name__} @ attribute {attr.name}'
-            exc.__note__ = note  # pyright: ignore
+            notes = [*getattr(exc, '__notes__', []), note]
+            exc.__note__ = note  # pyright: ignore  # TODO: Rm when new version of cattrs is out
+            exc.__notes__ = notes  # pyright: ignore
             raise exc
 
     return wrapper
 
 
 @_enrich_validator_exc
-def _validate_path_is_writable_dir(model: object, attr: Attribute[Path], value: Path):
+def _validate_path_is_writable_dir(_model: object, _attr: Attribute[Path], value: Path):
     if not _is_writable_dir(value):
         raise ValueError(f'"{value}" is not a writable directory')
 
 
 def _make_validate_min_length(min_length: int):
     @_enrich_validator_exc
-    def _validate_min_length(model: object, attr: Attribute[Sized], value: Sized):
+    def _validate_min_length(_model: object, _attr: Attribute[Sized], value: Sized):
         if len(value) < min_length:
             raise ValueError(f'value must have a minimum length of {min_length}')
 
@@ -74,29 +77,15 @@ def _make_validate_min_length(min_length: int):
 
 
 def _encode_config_for_display(config: object):
-    converter = make_config_converter()
-    converter.register_unstructure_hook(SecretStr, _convert_secret_string)
+    converter = _make_display_converter()
     return json.dumps(
         converter.unstructure(config),  # pyright: ignore[reportUnknownMemberType]
         indent=2,
     )
 
 
-def _write_config(config: object, config_path: Path, fields_to_include: Set[str]):
-    config_cls = config.__class__
-    converter = make_config_converter()
-    converter.register_unstructure_hook(
-        config_cls,
-        make_dict_unstructure_fn(  # pyright: ignore
-            config_cls,
-            converter,
-            **{
-                f.name: override(omit=True)
-                for f in fields(config_cls)
-                if f.name not in fields_to_include
-            },
-        ),
-    )
+def _write_config(config: object, config_path: Path, fields_to_include: frozenset[str]):
+    converter = _make_write_converter(config.__class__, fields_to_include)
     config_path.write_text(
         json.dumps(
             converter.unstructure(config), indent=2  # pyright: ignore[reportUnknownMemberType]
@@ -105,30 +94,29 @@ def _write_config(config: object, config_path: Path, fields_to_include: Set[str]
     )
 
 
-def _read_config(config_path: Path, missing_ok: bool = False):
+def _read_config(config_path: Path, missing_ok: bool = False) -> dict[str, Any]:
     try:
         return json.loads(config_path.read_bytes())
     except FileNotFoundError:
         if missing_ok:
-            default_config: dict[str, Any] = {}
-            return default_config
+            return {}
         raise
 
 
-def _compute_var(field: Attribute[object], default: object):
-    if not field.metadata.get('env'):
+def _compute_var(field_: Attribute[object], default: object):
+    if not field_.metadata.get('env'):
         return default
 
-    value = os.environ.get(f'instawow_{field.name}'.upper())
+    value = os.environ.get(f'instawow_{field_.name}'.upper())
     if value is None:
         return default
-    elif field.metadata.get('as_json'):
+    elif field_.metadata.get('as_json'):
         return json.loads(value)
     else:
         return value
 
 
-def _read_env_vars(config_cls: type, **values: object):
+def _read_env_vars(config_cls: Any, **values: object):
     return {
         f.name: v
         for f in fields(config_cls)
@@ -145,22 +133,56 @@ def make_config_converter():
     return converter
 
 
+@lru_cache(1)
+def _make_display_converter():
+    converter = make_config_converter()
+    converter.register_unstructure_hook(SecretStr, _convert_secret_string)
+    return converter
+
+
+@lru_cache(2)
+def _make_write_converter(config_cls: Any, fields_to_include: frozenset[str]):
+    converter = make_config_converter()
+    converter.register_unstructure_hook(
+        config_cls,
+        make_dict_unstructure_fn(  # pyright: ignore
+            config_cls,
+            converter,
+            **{
+                f.name: override(omit=True)
+                for f in fields(config_cls)
+                if f.name not in fields_to_include
+            },
+        ),
+    )
+    return converter
+
+
+def _get_default_config_dir():
+    return Path(click.get_app_dir('instawow'))
+
+
+def _get_default_temp_dir():
+    return Path(gettempdir(), 'instawow')
+
+
 @frozen
 class _AccessTokens:
+    cfcore: typing.Optional[SecretStr] = None
     github: typing.Optional[SecretStr] = None
     wago: typing.Optional[SecretStr] = None
-    cfcore: typing.Optional[SecretStr] = None
+    wago_addons: typing.Optional[SecretStr] = None
 
 
 @frozen
 class GlobalConfig:
     config_dir: Path = field(
-        factory=lambda: Path(click.get_app_dir('instawow')),
+        factory=_get_default_config_dir,
         converter=_expand_path,
         metadata={'env': True},
     )
     temp_dir: Path = field(
-        factory=lambda: Path(gettempdir(), 'instawow'),
+        factory=_get_default_temp_dir,
         converter=_expand_path,
         metadata={'env': True},
     )
@@ -181,7 +203,7 @@ class GlobalConfig:
     def read(cls) -> Self:
         env_only_config = cls.from_env()
         config_values = _read_config(env_only_config.config_file, missing_ok=True)
-        return cls.from_env(**config_values) if config_values is not None else env_only_config
+        return cls.from_env(**config_values) if config_values else env_only_config
 
     def list_profiles(self) -> list[str]:
         "Get the names of the profiles contained in ``config_dir``."
@@ -200,7 +222,7 @@ class GlobalConfig:
 
     def write(self) -> Self:
         self.ensure_dirs()
-        _write_config(self, self.config_file, {'auto_update_check', 'access_tokens'})
+        _write_config(self, self.config_file, frozenset({'auto_update_check', 'access_tokens'}))
         return self
 
     @property
@@ -257,7 +279,7 @@ class Config:
 
     def write(self) -> Self:
         self.ensure_dirs()
-        _write_config(self, self.config_file, {'addon_dir', 'game_flavour', 'profile'})
+        _write_config(self, self.config_file, frozenset({'addon_dir', 'game_flavour', 'profile'}))
         return self
 
     def delete(self) -> None:
@@ -345,10 +367,12 @@ def _intercept_logging_module_calls(log_level: str):  # pragma: no cover
     logging.basicConfig(handlers=[InterceptHandler()], level=log_level)
 
 
-def setup_logging(logging_dir: Path, debug: bool) -> None:
+def setup_logging(
+    logging_dir: Path, log_to_stderr: bool, debug: bool, intercept_logging_module_calls: bool
+) -> None:
     log_level = 'DEBUG' if debug else 'INFO'
 
-    if debug:
+    if intercept_logging_module_calls:
         _intercept_logging_module_calls(log_level)
 
     _patch_loguru_enqueue()
@@ -367,7 +391,7 @@ def setup_logging(logging_dir: Path, debug: bool) -> None:
             },
         ]
     )
-    if debug:
+    if log_to_stderr:
         logger.add(
             **values,
             sink=sys.stderr,
