@@ -2,83 +2,86 @@ from __future__ import annotations
 
 import asyncio
 from collections import defaultdict
-from collections.abc import AsyncIterator, Awaitable, Callable, Iterable, Mapping, Sequence, Set
+from collections.abc import Awaitable, Callable, Collection, Iterable, Mapping, Sequence, Set
 from contextlib import AbstractAsyncContextManager, asynccontextmanager, contextmanager
 import contextvars as cv
-from datetime import datetime
-from functools import lru_cache, partial, wraps
-from itertools import chain, filterfalse, repeat, starmap, takewhile
+from datetime import datetime, timedelta
+from functools import wraps
+from itertools import chain, filterfalse, product, repeat, starmap, takewhile
 import json
 from pathlib import Path, PurePath
 from shutil import copy
 from tempfile import NamedTemporaryFile
-import time
-from typing import Any, TypeVar
+import typing
+from typing import NoReturn, TypeVar
 
-from attrs import evolve, fields
+from attrs import evolve
 from loguru import logger
 import sqlalchemy as sa
 import sqlalchemy.future as sa_future
-from typing_extensions import Concatenate, Literal, ParamSpec, TypeAlias, TypedDict
+from typing_extensions import Concatenate, ParamSpec, TypeAlias
 from yarl import URL
 
 from . import _deferred_types, db, models, results as R
-from .cataloguer import BaseCatalogue, Catalogue, CatalogueEntry, catalogue_converter
+from ._sources.cfcore import CfCoreResolver
+from ._sources.github import GithubResolver
+from ._sources.instawow import InstawowResolver
+from ._sources.tukui import TukuiResolver
+from ._sources.wago import WagoResolver
+from ._sources.wowi import WowiResolver
+from .cataloguer import (
+    BASE_CATALOGUE_VERSION,
+    CATALOGUE_VERSION,
+    Catalogue,
+    CatalogueEntry,
+    catalogue_converter,
+)
 from .common import Strategy
 from .config import Config, GlobalConfig
-from .models import PkgFolder
+from .http import make_generic_progress_ctx, make_pkg_progress_ctx
 from .plugins import load_plugins
-from .resolvers import (
-    CfCoreResolver,
-    Defn,
-    GithubResolver,
-    InstawowResolver,
-    Resolver,
-    TukuiResolver,
-    WowiResolver,
-)
+from .resolvers import Defn, Resolver
 from .utils import (
     bucketise,
+    cached_property,
     chain_dict,
     file_uri_to_path,
     find_addon_zip_tocs,
     gather,
+    is_file_uri,
     is_not_stale,
-    make_zip_member_filter,
+    make_zip_member_filter_fn,
     move,
     normalise_names,
     run_in_thread as t,
     shasum,
+    time_op,
     trash,
     uniq,
 )
 
 _P = ParamSpec('_P')
 _T = TypeVar('_T')
-
-
-USER_AGENT = 'instawow (https://github.com/layday/instawow)'
+_ResultOrError: TypeAlias = '_T | R.ManagerError | R.InternalError'
 
 DB_REVISION = 'e13430219249'
 
-
-class _GenericDownloadTraceRequestCtx(TypedDict):
-    report_progress: Literal['generic']
-    label: str
-
-
-class _PkgDownloadTraceRequestCtx(TypedDict):
-    report_progress: Literal['pkg_download']
-    manager: Manager
-    pkg: models.Pkg
-
-
-TraceRequestCtx: TypeAlias = '_GenericDownloadTraceRequestCtx | _PkgDownloadTraceRequestCtx | None'
-
-
 _move_async = t(move)
 
 
+_ERROR_CLASSES = (R.ManagerError, R.InternalError)
+
+
+def _bucketise_results(
+    value: Iterable[tuple[Defn, _ResultOrError[_T]]],
+) -> tuple[Mapping[Defn, _T], Mapping[Defn, _ResultOrError[NoReturn]]]:
+    def get_bucket_dict(key: bool):
+        return dict(buckets.get(key, ()))
+
+    buckets = bucketise(value, lambda v: isinstance(v[1], _ERROR_CLASSES))
+    return (get_bucket_dict(False), get_bucket_dict(True))  # pyright: ignore
+
+
 @asynccontextmanager
 async def _open_temp_writer():
     loop = asyncio.get_running_loop()
@@ -101,138 +104,25 @@ def _open_pkg_archive(path: PurePath):
     with ZipFile(path) as archive:
 
         def extract(parent: Path) -> None:
-            archive.extractall(parent, members=filter(make_zip_member_filter(base_dirs), names))
+            archive.extractall(parent, members=filter(make_zip_member_filter_fn(base_dirs), names))
 
         names = archive.namelist()
         base_dirs = {h for _, h in find_addon_zip_tocs(names)}
         yield (base_dirs, extract)
 
 
-class _ResponseWrapper:
-    def __init__(
-        self, response: _deferred_types.aiohttp.ClientResponse | None, response_body: bytes
-    ):
-        self._response = response
-        self._response_body = response_body
-
-    async def read(self) -> bytes:
-        return self._response_body
-
-    async def text(self) -> str:
-        return self._response_body.decode()
-
-    async def json(self, **kwargs: Any) -> Any:
-        return json.loads(await self.read())
-
-    def raise_for_status(self) -> None:
-        if self._response is not None:
-            self._response.raise_for_status()
-
-    @property
-    def status(self) -> int:
-        return self._response.status if self._response is not None else 200
-
-
-class _CacheFauxClientSession:
-    def __init__(self, cache_dir: Path):
-        self._cache_dir = cache_dir
-
-    @property
-    def wrapped(self) -> _deferred_types.aiohttp.ClientSession:
-        return _web_client.get()
-
-    @asynccontextmanager
-    async def request(
-        self,
-        method: str,
-        url: str | URL,
-        ttl: Mapping[str, float] | None = None,
-        label: str | None = None,
-        **kwargs: Any,
-    ) -> AsyncIterator[_ResponseWrapper]:
-        def prepare_request_kwargs():
-            request_kwargs: dict[str, Any] = {'method': method, 'url': url, **kwargs}
-            if label:
-                request_kwargs['trace_request_ctx'] = _GenericDownloadTraceRequestCtx(
-                    report_progress='generic', label=label
-                )
-            return request_kwargs
-
-        async def make_request():
-            async with self.wrapped.request(**prepare_request_kwargs()) as response:
-                return (response, await response.read())
-
-        if ttl is None:
-            response, response_body = await make_request()
-        else:
-            dest = self._cache_dir / shasum(url, ttl, kwargs)
-            response = None
-            if await t(is_not_stale)(dest, ttl):
-                logger.debug(f'{url} is cached at {dest} (ttl: {ttl})')
-                response_body = await t(dest.read_bytes)()
-            else:
-                response, response_body = await make_request()
-                if response.ok:
-                    await t(dest.write_bytes)(response_body)
-
-        yield _ResponseWrapper(response, response_body)
-
-    def get(
-        self,
-        url: str | URL,
-        ttl: Mapping[str, float] | None = None,
-        label: str | None = None,
-        **kwargs: Any,
-    ) -> AbstractAsyncContextManager[_ResponseWrapper]:
-        return self.request('GET', url, ttl, label, **kwargs)
-
-
-@lru_cache(None)
-def _load_certifi_certs():
-    try:
-        import certifi
-    except ModuleNotFoundError:
-        pass
-    else:
-        from importlib.resources import read_text
-
-        logger.info('loading certifi certs')
-        return read_text(certifi, 'cacert.pem', encoding='ascii')
-
-
-def init_web_client(**kwargs: Any) -> _deferred_types.aiohttp.ClientSession:
-    from aiohttp import ClientSession, ClientTimeout, TCPConnector
-
-    make_connector = partial(TCPConnector, limit_per_host=10)
-    certifi_certs = _load_certifi_certs()
-    if certifi_certs:
-        import ssl
-
-        make_connector = partial(
-            make_connector, ssl=ssl.create_default_context(cadata=certifi_certs)
-        )
-
-    kwargs = {
-        'connector': make_connector(),
-        'headers': {'User-Agent': USER_AGENT},
-        'trust_env': True,  # Respect the 'http_proxy' env var
-        'timeout': ClientTimeout(connect=60, sock_connect=10, sock_read=20),
-        **kwargs,
-    }
-    return ClientSession(**kwargs)
-
-
 @object.__new__
 class _DummyResolver:
-    async def resolve(
-        self, defns: Sequence[Defn]
-    ) -> dict[Defn, models.Pkg | R.ManagerError | R.InternalError]:
+    async def resolve(self, defns: Sequence[Defn]) -> dict[Defn, _ResultOrError[models.Pkg]]:
         return dict.fromkeys(defns, R.PkgSourceInvalid())
 
+    async def get_changelog(self, uri: URL) -> str:
+        raise R.PkgSourceInvalid
+
 
 async def capture_manager_exc_async(
     awaitable: Awaitable[_T],
-) -> _T | R.ManagerError | R.InternalError:
+) -> _ResultOrError[_T]:
     "Capture and log an exception raised in a coroutine."
     from aiohttp import ClientError
 
@@ -273,6 +163,28 @@ def _with_lock(
     return outer
 
 
+class _Resolvers(typing.Dict[str, Resolver]):
+    @cached_property
+    def priority_dict(self) -> _ResolverPriorityDict:
+        return _ResolverPriorityDict(self)
+
+    @cached_property
+    def addon_toc_key_and_id_pairs(self) -> Collection[tuple[str, str]]:
+        return [
+            (r.metadata.addon_toc_key, r.metadata.id)
+            for r in self.values()
+            if r.metadata.addon_toc_key
+        ]
+
+
+class _ResolverPriorityDict(typing.Dict[str, float]):
+    def __init__(self, resolvers: _Resolvers) -> None:
+        super().__init__((n, i) for i, n in enumerate(resolvers))
+
+    def __missing__(self, key: str) -> float:
+        return float('inf')
+
+
 _web_client: cv.ContextVar[_deferred_types.aiohttp.ClientSession] = cv.ContextVar('_web_client')
 
 LocksType: TypeAlias = 'defaultdict[object, AbstractAsyncContextManager[None]]'
@@ -300,14 +212,15 @@ class Manager:
         WowiResolver,
         TukuiResolver,
         InstawowResolver,
+        WagoResolver,
     ]
     "Default resolvers."
 
     _base_catalogue_url = (
         f'https://raw.githubusercontent.com/layday/instawow-data/data/'
-        f'base-catalogue-v{fields(BaseCatalogue).version.default}.compact.json'
+        f'base-catalogue-v{BASE_CATALOGUE_VERSION}.compact.json'
     )
-    _catalogue_filename = f'catalogue-v{fields(Catalogue).version.default}.json'
+    _catalogue_filename = f'catalogue-v{CATALOGUE_VERSION}.json'
 
     _normalise_search_terms = staticmethod(normalise_names(''))
 
@@ -320,18 +233,20 @@ class Manager:
         self.database: sa_future.Connection = database
 
         builtin_resolver_classes = list(self.RESOLVERS)
-        if self.config.global_config.access_tokens.cfcore is None:
-            builtin_resolver_classes.remove(CfCoreResolver)
+
+        for resolver, access_token in (
+            (r, getattr(self.config.global_config.access_tokens, r.requires_access_token, None))
+            for r in self.RESOLVERS
+            if r.requires_access_token is not None
+        ):
+            if access_token is None:
+                builtin_resolver_classes.remove(resolver)
 
         plugin_hook = load_plugins()
         resolver_classes = chain(
             (r for g in plugin_hook.instawow_add_resolvers() for r in g), builtin_resolver_classes
         )
-        self.resolvers: Mapping[str, Resolver] = {r.metadata.id: r(self) for r in resolver_classes}
-
-        self.web_client: _CacheFauxClientSession = _CacheFauxClientSession(
-            self.config.global_config.cache_dir
-        )
+        self.resolvers = _Resolvers((r.metadata.id, r(self)) for r in resolver_classes)
 
         self._catalogue = None
 
@@ -346,6 +261,10 @@ class Manager:
         "Lock factory used to synchronise async operations."
         return _locks.get()
 
+    @property
+    def web_client(self) -> _deferred_types.aiohttp.ClientSession:
+        return _web_client.get()
+
     def pair_uri(self, value: str) -> tuple[str, str] | None:
         "Attempt to extract the definition source and alias from a URI."
 
@@ -405,7 +324,8 @@ class Manager:
         if maybe_row_mapping is not None:
             return models.Pkg.from_row_mapping(self.database, maybe_row_mapping)
 
-    def install_pkg(self, pkg: models.Pkg, archive: Path, replace: bool) -> R.PkgInstalled:
+    @t
+    def _install_pkg(self, pkg: models.Pkg, archive: Path, replace: bool) -> R.PkgInstalled:
         "Install a package."
         with _open_pkg_archive(archive) as (top_level_folders, extract):
             installed_conflicts = self.database.execute(
@@ -432,11 +352,12 @@ class Manager:
 
             extract(self.config.addon_dir)
 
-        pkg = evolve(pkg, folders=[PkgFolder(name=f) for f in sorted(top_level_folders)])
+        pkg = evolve(pkg, folders=[models.PkgFolder(name=f) for f in sorted(top_level_folders)])
         pkg.insert(self.database)
         return R.PkgInstalled(pkg)
 
-    def update_pkg(self, old_pkg: models.Pkg, new_pkg: models.Pkg, archive: Path) -> R.PkgUpdated:
+    @t
+    def _update_pkg(self, old_pkg: models.Pkg, new_pkg: models.Pkg, archive: Path) -> R.PkgUpdated:
         "Update a package."
         with _open_pkg_archive(archive) as (top_level_folders, extract):
             installed_conflicts = self.database.execute(
@@ -467,11 +388,14 @@ class Manager:
 
             extract(self.config.addon_dir)
 
-        new_pkg = evolve(new_pkg, folders=[PkgFolder(name=f) for f in sorted(top_level_folders)])
+        new_pkg = evolve(
+            new_pkg, folders=[models.PkgFolder(name=f) for f in sorted(top_level_folders)]
+        )
         new_pkg.insert(self.database)
         return R.PkgUpdated(old_pkg, new_pkg)
 
-    def remove_pkg(self, pkg: models.Pkg, keep_folders: bool) -> R.PkgRemoved:
+    @t
+    def _remove_pkg(self, pkg: models.Pkg, keep_folders: bool) -> R.PkgRemoved:
         "Remove a package."
         if not keep_folders:
             trash(
@@ -487,22 +411,18 @@ class Manager:
     async def synchronise(self) -> Catalogue:
         "Fetch the catalogue from the interwebs and load it."
         catalogue_json = self.config.global_config.temp_dir / self._catalogue_filename
-        if await t(is_not_stale)(catalogue_json, {'hours': 4}):
+        if await t(is_not_stale)(catalogue_json, timedelta(hours=4)):
             if self._catalogue is None:
-                raw_catalogue = await t(catalogue_json.read_bytes)()
-                start = time.perf_counter()
-                # Skip validation when loading the catalogue from cache.
-                self._catalogue = catalogue_converter.structure(
-                    json.loads(raw_catalogue), Catalogue
-                )
-                logger.debug(f'loaded catalogue from cache in {time.perf_counter() - start:.3f}s')
+                with time_op(lambda t: logger.debug(f'loaded catalogue from cache in {t:.3f}s')):
+                    raw_catalogue = await t(catalogue_json.read_bytes)()
+                    self._catalogue = catalogue_converter.structure(
+                        json.loads(raw_catalogue), Catalogue
+                    )
         else:
             async with self.web_client.get(
                 self._base_catalogue_url,
                 raise_for_status=True,
-                trace_request_ctx=_GenericDownloadTraceRequestCtx(
-                    report_progress='generic', label='Synchronising catalogue'
-                ),
+                trace_request_ctx=make_generic_progress_ctx('Synchronising catalogue'),
             ) as response:
                 raw_catalogue = await response.json(content_type=None)
 
@@ -519,13 +439,49 @@ class Manager:
         return self._catalogue
 
     async def find_equivalent_pkg_defns(
-        self, pkgs: Iterable[models.Pkg]
+        self, pkgs: Collection[models.Pkg]
     ) -> dict[models.Pkg, list[Defn]]:
         "Given a list of packages, find ``Defn``s of each package from other sources."
         from .matchers import AddonFolder
 
         catalogue = await self.synchronise()
 
+        @t
+        def collect_addon_folders():
+            return {
+                p: frozenset(
+                    a
+                    for f in p.folders
+                    for a in (
+                        AddonFolder.from_addon_path(
+                            self.config.game_flavour, self.config.addon_dir / f.name
+                        ),
+                    )
+                    if a
+                )
+                for p in pkgs
+            }
+
+        async def collect_hashed_folder_defns():
+            matches = [
+                m
+                for r in self.resolvers.values()
+                for m in await r.get_folder_hash_matches(
+                    [a for p, f in folders_per_pkg.items() if p.source != r.metadata.id for a in f]
+                )
+            ]
+
+            defns_per_pkg: dict[models.Pkg, frozenset[Defn]] = {
+                p: frozenset() for p in folders_per_pkg
+            }
+            for (pkg, orig_folders), (defn, matched_folders) in product(
+                folders_per_pkg.items(), matches
+            ):
+                if orig_folders == matched_folders:
+                    defns_per_pkg[pkg] |= frozenset((defn,))
+
+            return defns_per_pkg
+
         def get_catalogue_defns(pkg: models.Pkg) -> frozenset[Defn]:
             entry = catalogue.keyed_entries.get((pkg.source, pkg.id))
             if entry:
@@ -533,30 +489,33 @@ class Manager:
             else:
                 return frozenset()
 
-        def extract_addon_toc_defns(pkg: models.Pkg):
+        def get_addon_toc_defns(pkg_source: str, addon_folders: Collection[AddonFolder]):
             return frozenset(
                 d
-                for f in pkg.folders
-                for a in (
-                    AddonFolder.from_addon_path(
-                        self.config.game_flavour, self.config.addon_dir / f.name
-                    ),
-                )
-                if a
-                for d in a.defns_from_toc
-                if d.source != pkg.source
+                for a in addon_folders
+                for d in a.get_defns_from_toc_keys(self.resolvers.addon_toc_key_and_id_pairs)
+                if d.source != pkg_source
             )
 
-        source_priority = {n: i for i, n in enumerate(self.resolvers)}
+        folders_per_pkg = await collect_addon_folders()
+
+        with time_op(lambda t: logger.debug(f'hashed folder matches found in {t:.3f}s')):
+            hashed_folder_defns = await collect_hashed_folder_defns()
 
         return {
-            p: sorted(d, key=lambda d: source_priority.get(d.source, float('inf')))
+            p: sorted(d, key=lambda d: self.resolvers.priority_dict[d.source])
             for p in pkgs
-            for d in (get_catalogue_defns(p) | await t(extract_addon_toc_defns)(p),)
+            for d in (
+                get_catalogue_defns(p)
+                | get_addon_toc_defns(p.source, folders_per_pkg[p])
+                | hashed_folder_defns[p],
+            )
             if d
         }
 
-    async def _resolve_deps(self, results: Iterable[Any]) -> dict[Defn, Any]:
+    async def _resolve_deps(
+        self, results: Collection[_ResultOrError[models.Pkg]]
+    ) -> Mapping[Defn, _ResultOrError[models.Pkg]]:
         """Resolve package dependencies.
 
         The resolver will not follow dependencies
@@ -581,8 +540,8 @@ class Manager:
         return pretty_deps
 
     async def resolve(
-        self, defns: Sequence[Defn], with_deps: bool = False
-    ) -> dict[Defn, models.Pkg | R.ManagerError | R.InternalError]:
+        self, defns: Collection[Defn], with_deps: bool = False
+    ) -> Mapping[Defn, _ResultOrError[models.Pkg]]:
         "Resolve definitions into packages."
         if not defns:
             return {}
@@ -592,9 +551,9 @@ class Manager:
             (self.resolvers.get(s, _DummyResolver).resolve(b) for s, b in defns_by_source.items()),
             capture_manager_exc_async,
         )
-        results_by_defn: dict[Defn, Any] = chain_dict(
+        results_by_defn = chain_dict(
             defns,
-            None,
+            R.ManagerError(),
             *(
                 r.items() if isinstance(r, dict) else zip(d, repeat(r))
                 for d, r in zip(defns_by_source.values(), results)
@@ -606,9 +565,7 @@ class Manager:
 
     async def get_changelog(self, source: str, uri: str) -> str:
         "Retrieve a changelog from a URI."
-        if source not in self.resolvers:
-            raise R.PkgSourceInvalid
-        return await self.resolvers[source].get_changelog(URL(uri))
+        return await self.resolvers.get(source, _DummyResolver).get_changelog(URL(uri))
 
     async def search(
         self,
@@ -619,8 +576,7 @@ class Manager:
         installed_only: bool = False,
     ) -> list[CatalogueEntry]:
         "Search the master catalogue for packages by name."
-        import rapidfuzz.fuzz
-        import rapidfuzz.process
+        import rapidfuzz
 
         catalogue = await self.synchronise()
 
@@ -636,7 +592,7 @@ class Manager:
             if unknown_sources:
                 raise ValueError(f'Unknown sources: {", ".join(unknown_sources)}')
 
-        def make_filter():
+        def make_filter_fns():
             def filter_game_flavour(entry: CatalogueEntry):
                 return self.config.game_flavour in entry.game_flavours
 
@@ -657,7 +613,7 @@ class Manager:
 
                 yield filter_age
 
-        filter_fns = list(make_filter())
+        filter_fns = list(make_filter_fns())
 
         s = self._normalise_search_terms(search_terms)
 
@@ -693,60 +649,76 @@ class Manager:
         return [e for _, e in weighted_entries[:limit]]
 
     async def _download_pkg_archive(self, pkg: models.Pkg, *, chunk_size: int = 4096):
-        url = pkg.download_url
-        dest = self.config.global_config.cache_dir / shasum(url)
+        dest = self.config.global_config.cache_dir / shasum(pkg.download_url)
 
         if await t(dest.exists)():
-            logger.debug(f'{url} is cached at {dest}')
-        elif url.startswith('file://'):
+            pass
+
+        elif is_file_uri(pkg.download_url):
             loop = asyncio.get_running_loop()
-            await loop.run_in_executor(None, lambda: copy(file_uri_to_path(url), dest))
+            await loop.run_in_executor(
+                None, lambda: copy(file_uri_to_path(pkg.download_url), dest)
+            )
+
         else:
-            async with self.web_client.wrapped.get(
-                url,
+            async with self.web_client.get(
+                pkg.download_url,
+                headers=await self.resolvers[pkg.source].make_auth_headers(),
                 raise_for_status=True,
-                trace_request_ctx=_PkgDownloadTraceRequestCtx(
-                    report_progress='pkg_download', manager=self, pkg=pkg
-                ),
-            ) as response, _open_temp_writer() as (temp_path, write):
+                trace_request_ctx=make_pkg_progress_ctx(self.config.profile, pkg),
+            ) as response, _open_temp_writer() as (
+                temp_path,
+                write,
+            ):
                 async for chunk in response.content.iter_chunked(chunk_size):
                     await write(chunk)
-
             await _move_async(temp_path, dest)
 
         return dest
 
+    @t
+    def _check_installed_pkg_integrity(self, pkg: models.Pkg) -> bool:
+        return all((self.config.addon_dir / p.name).exists() for p in pkg.folders)
+
+    async def _should_update_pkg(self, old_pkg: models.Pkg, new_pkg: models.Pkg) -> bool:
+        return old_pkg.version != new_pkg.version or (
+            not await self._check_installed_pkg_integrity(old_pkg)
+        )
+
     @_with_lock('change state')
     async def install(
         self, defns: Sequence[Defn], replace: bool
-    ) -> dict[Defn, R.PkgInstalled | R.ManagerError | R.InternalError]:
+    ) -> Mapping[Defn, _ResultOrError[R.PkgInstalled]]:
         "Install packages from a definition list."
         # We'll weed out installed deps from the results after resolving -
         # doing it this way isn't particularly efficient but avoids having to
-        # deal with local state in `resolve()`
+        # deal with local state in ``resolve``
         resolve_results = await self.resolve(
             [d for d in defns if not self.check_pkg_exists(d)], with_deps=True
         )
-        resolve_results = {
-            d: r for d, r in resolve_results.items() if not self.check_pkg_exists(d)
-        }
-        installables = {d: r for d, r in resolve_results.items() if isinstance(r, models.Pkg)}
-        archives = await gather(
-            (self._download_pkg_archive(r) for r in installables.values()),
-            capture_manager_exc_async,
+        pkgs, resolve_errors = _bucketise_results(
+            (d, r) for d, r in resolve_results.items() if not self.check_pkg_exists(d)
         )
-        results: dict[Defn, Any] = chain_dict(
+        archive_paths, download_errors = _bucketise_results(
+            zip(
+                pkgs,
+                await gather(
+                    (self._download_pkg_archive(r) for r in pkgs.values()),
+                    capture_manager_exc_async,
+                ),
+            )
+        )
+        results = chain_dict(
             defns,
             R.PkgAlreadyInstalled(),
-            resolve_results.items(),
+            resolve_errors.items(),
+            download_errors.items(),
             [
                 (
                     d,
-                    await capture_manager_exc_async(t(self.install_pkg)(p, a, replace))
-                    if isinstance(a, PurePath)
-                    else a,
+                    await capture_manager_exc_async(self._install_pkg(pkgs[d], a, replace)),
                 )
-                for (d, p), a in zip(installables.items(), archives)
+                for d, a in archive_paths.items()
             ],
         )
         return results
@@ -754,7 +726,7 @@ class Manager:
     @_with_lock('change state')
     async def update(
         self, defns: Sequence[Defn], retain_defn_strategy: bool
-    ) -> dict[Defn, R.PkgUpdated | R.ManagerError | R.InternalError]:
+    ) -> Mapping[Defn, _ResultOrError[R.PkgInstalled | R.PkgUpdated]]:
         """Update installed packages from a definition list.
 
         A ``retain_defn_strategy`` value of false will instruct ``update``
@@ -770,37 +742,45 @@ class Manager:
             evolve(d, id=p.id) if retain_defn_strategy else Defn.from_pkg(p): d
             for d, p in defns_to_pkgs.items()
         }
+        resolve_results = await self.resolve(resolve_defns, with_deps=True)
         # Discard the reconstructed ``Defn``s
-        resolve_results = {
-            resolve_defns[d]: r for d, r in (await self.resolve(list(resolve_defns))).items()
-        }
-        installables = {d: r for d, r in resolve_results.items() if isinstance(r, models.Pkg)}
+        orig_defn_resolve_results = (
+            (resolve_defns.get(d, d), r) for d, r in resolve_results.items()
+        )
+        pkgs, resolve_errors = _bucketise_results(orig_defn_resolve_results)
         updatables = {
             d: (o, n)
-            for d, n in installables.items()
-            for o in (defns_to_pkgs[d],)
-            if n.version != o.version
+            for d, n in pkgs.items()
+            for o in (defns_to_pkgs.get(d),)
+            if not o or await self._should_update_pkg(o, n)
         }
-        archives = await gather(
-            (self._download_pkg_archive(n) for _, n in updatables.values()),
-            capture_manager_exc_async,
+        archive_paths, download_errors = _bucketise_results(
+            zip(
+                updatables,
+                await gather(
+                    (self._download_pkg_archive(n) for _, n in updatables.values()),
+                    capture_manager_exc_async,
+                ),
+            )
         )
-        results: dict[Defn, Any] = chain_dict(
+        results = chain_dict(
             defns,
             R.PkgNotInstalled(),
-            resolve_results.items(),
+            resolve_errors.items(),
             (
-                (d, R.PkgUpToDate(is_pinned=p.options.strategy == Strategy.version))
-                for d, p in installables.items()
+                (d, R.PkgUpToDate(is_pinned=pkgs[d].options.strategy is Strategy.version))
+                for d in pkgs.keys() - updatables.keys()
             ),
+            download_errors.items(),
             [
                 (
                     d,
-                    await capture_manager_exc_async(t(self.update_pkg)(o, n, a))
-                    if isinstance(a, PurePath)
-                    else a,
+                    await capture_manager_exc_async(
+                        self._update_pkg(o, n, a) if o else self._install_pkg(n, a, False)
+                    ),
                 )
-                for (d, (o, n)), a in zip(updatables.items(), archives)
+                for d, a in archive_paths.items()
+                for o, n in (updatables[d],)
             ],
         )
         return results
@@ -808,31 +788,20 @@ class Manager:
     @_with_lock('change state')
     async def remove(
         self, defns: Sequence[Defn], keep_folders: bool
-    ) -> dict[Defn, R.PkgRemoved | R.PkgNotInstalled | R.ManagerError | R.InternalError]:
+    ) -> Mapping[Defn, _ResultOrError[R.PkgRemoved]]:
         "Remove packages by their definition."
-        results = chain_dict(
-            defns,
-            R.PkgNotInstalled(),
-            [
-                (d, await capture_manager_exc_async(t(self.remove_pkg)(p, keep_folders)))
-                for d in defns
-                for p in (self.get_pkg(d),)
+        return {
+            d: (
+                await capture_manager_exc_async(self._remove_pkg(p, keep_folders))
                 if p
-            ],
-        )
-        return results
+                else R.PkgNotInstalled()
+            )
+            for d in defns
+            for p in (self.get_pkg(d),)
+        }
 
     @_with_lock('change state')
-    async def pin(
-        self, defns: Sequence[Defn]
-    ) -> dict[
-        Defn,
-        R.PkgNotInstalled
-        | R.PkgInstalled
-        | R.PkgStrategyUnsupported
-        | R.ManagerError
-        | R.InternalError,
-    ]:
+    async def pin(self, defns: Sequence[Defn]) -> Mapping[Defn, _ResultOrError[R.PkgInstalled]]:
         """Pin and unpin installed packages.
 
         instawow does not have true pinning.  This sets the strategy
@@ -843,66 +812,65 @@ class Manager:
         package.
         """
 
-        strategies = frozenset({Strategy.default, Strategy.version})
+        PINNING_STRATEGIES = frozenset({Strategy.default, Strategy.version})
 
-        def pin(defn: Defn, pkg: models.Pkg | None) -> R.PkgInstalled:
+        @t
+        def pin(defn: Defn) -> R.PkgInstalled:
+            pkg = self.get_pkg(defn)
             if not pkg:
                 raise R.PkgNotInstalled
 
-            elif {defn.strategy} <= strategies <= self.resolvers[pkg.source].metadata.strategies:
-                self.database.execute(
-                    sa.update(db.pkg_options)
-                    .filter_by(pkg_source=pkg.source, pkg_id=pkg.id)
-                    .values(strategy=defn.strategy)
-                )
-                self.database.commit()
-                row_mapping = (
+            resolver = self.resolvers.get(pkg.source)
+            if resolver is None:
+                raise R.PkgSourceInvalid
+
+            if {defn.strategy} <= PINNING_STRATEGIES <= resolver.metadata.strategies:
+                with db.faux_transact(self.database):
                     self.database.execute(
-                        sa.select(db.pkg).filter_by(source=pkg.source, id=pkg.id)
+                        sa.update(db.pkg_options)
+                        .filter_by(pkg_source=pkg.source, pkg_id=pkg.id)
+                        .values(strategy=defn.strategy)
                     )
-                    .mappings()
-                    .one()
-                )
-                return R.PkgInstalled(models.Pkg.from_row_mapping(self.database, row_mapping))
+
+                new_pkg = evolve(pkg, options=evolve(pkg.options, strategy=defn.strategy))
+                return R.PkgInstalled(new_pkg)
 
             else:
                 raise R.PkgStrategyUnsupported(Strategy.version)
 
-        return {d: await capture_manager_exc_async(t(pin)(d, self.get_pkg(d))) for d in defns}
+        return {d: await capture_manager_exc_async(pin(d)) for d in defns}
 
 
-async def is_outdated() -> tuple[bool, str]:
+async def is_outdated(global_config: GlobalConfig) -> tuple[bool, str]:
     """Check on PyPI to see if instawow is outdated.
 
     The response is cached for 24 hours.
     """
-    from aiohttp.client import ClientError
+
+    if not global_config.auto_update_check:
+        return (False, '')
 
     from . import __version__
 
+    if __version__ == '0+dev':
+        return (False, '')
+
+    from aiohttp.client import ClientError
+
+    from .http import init_web_client
+
     def parse_version(version: str):
         version_parts = takewhile(lambda p: all(c in '0123456789' for c in p), version.split('.'))
         return tuple(map(int, version_parts))
 
-    if __version__ == '0.0.0':
-        return (False, '')
-
-    global_config = await t(GlobalConfig.read)()
-    if not global_config.auto_update_check:
-        return (False, '')
-
-    cache_file = global_config.temp_dir / '.pypi_version'
-    if await t(is_not_stale)(cache_file, {'days': 1}):
-        version = cache_file.read_text(encoding='utf-8')
-    else:
-        try:
-            async with init_web_client(raise_for_status=True) as web_client, web_client.get(
-                'https://pypi.org/pypi/instawow/json'
-            ) as response:
-                version = (await response.json())['info']['version']
-        except ClientError:
-            version = __version__
-        else:
-            cache_file.write_text(version, encoding='utf-8')
+    try:
+        async with init_web_client(
+            global_config.cache_dir, raise_for_status=True
+        ) as web_client, web_client.get(
+            'https://pypi.org/pypi/instawow/json', expire_after=timedelta(days=1)
+        ) as response:
+            version = (await response.json())['info']['version']
+    except ClientError:
+        version = __version__
 
     return (parse_version(version) > parse_version(__version__), version)
