@@ -2,10 +2,11 @@ from __future__ import annotations
 
 import asyncio
 from collections import defaultdict
-from collections.abc import Awaitable, Callable, Iterable, Iterator, Mapping, Sequence, Set
+from collections.abc import Awaitable, Callable, Collection, Iterable, Iterator, Sequence, Set
+from contextlib import contextmanager
 from datetime import datetime, timedelta
-import enum
 from functools import partial, wraps
+import importlib.resources
 from itertools import chain, groupby, islice, repeat
 import os
 from pathlib import Path, PurePath
@@ -13,6 +14,8 @@ import posixpath
 from shutil import move as _move
 import sys
 from tempfile import mkdtemp
+import time
+from types import ModuleType
 from typing import Any, Generic, Hashable, TypeVar, overload
 
 from typing_extensions import ParamSpec
@@ -23,8 +26,52 @@ _THashable = TypeVar('_THashable', bound=Hashable)
 _P = ParamSpec('_P')
 
 
-class StrEnum(str, enum.Enum):
-    pass
+def as_decorated_type(__type: type[_T]) -> Callable[[_T], _T]:
+    return lambda v: v
+
+
+if sys.version_info >= (3, 11):
+    from enum import StrEnum as StrEnum
+else:
+    from enum import Enum
+
+    class StrEnum(str, Enum):
+        pass
+
+
+if sys.version_info >= (3, 9):
+
+    def read_resource_as_text(package: ModuleType, resource: str, encoding: str = 'utf-8') -> str:
+        return importlib.resources.files(package).joinpath(resource).read_text(encoding)
+
+else:
+
+    def read_resource_as_text(package: ModuleType, resource: str, encoding: str = 'utf-8') -> str:
+        return importlib.resources.read_text(package, resource, encoding=encoding)
+
+
+if sys.version_info >= (3, 8):
+    from functools import cached_property as cached_property
+else:
+
+    class cached_property(Generic[_T, _U]):
+        def __init__(self, f: Callable[[_T], _U]) -> None:
+            self.f = f
+
+        @overload
+        def __get__(self, o: None, t: type[_T] | None = ...) -> cached_property[_T, _U]:
+            ...
+
+        @overload
+        def __get__(self, o: _T, t: type[_T] | None = ...) -> _U:
+            ...
+
+        def __get__(self, o: _T | None, t: type[_T] | None = None) -> cached_property[_T, _U] | _U:
+            if o is None:
+                return self
+            else:
+                o.__dict__[self.f.__name__] = v = self.f(o)
+                return v
 
 
 class TocReader:
@@ -46,33 +93,13 @@ class TocReader:
             return self.entries.get(key)
 
     @classmethod
-    def from_addon_path(cls, path: Path, suffix: str = '.toc') -> TocReader:
-        return cls((path / (path.name + suffix)).read_text(encoding='utf-8-sig', errors='replace'))
+    def from_path(cls, path: Path) -> TocReader:
+        return cls(path.read_text(encoding='utf-8-sig', errors='replace'))
 
 
-class cached_property(Generic[_T, _U]):
-    def __init__(self, f: Callable[[_T], _U]) -> None:
-        self.f = f
-
-    @overload
-    def __get__(self, o: None, t: type[_T] | None = ...) -> cached_property[_T, _U]:
-        ...
-
-    @overload
-    def __get__(self, o: _T, t: type[_T] | None = ...) -> _U:
-        ...
-
-    def __get__(self, o: _T | None, t: type[_T] | None = None) -> cached_property[_T, _U] | _U:
-        if o is None:
-            return self
-        else:
-            o.__dict__[self.f.__name__] = v = self.f(o)
-            return v
-
-
-def fill(it: Iterable[_T], fill: _T, number: int) -> Iterable[_T]:
-    "Append entries to a sized iterable."
-    return islice(chain(it, repeat(fill)), 0, number)
+def fill(it: Iterable[_T], fill: _T, length: int) -> Iterable[_T]:
+    "Fill an iterable of specified length."
+    return islice(chain(it, repeat(fill)), 0, length)
 
 
 def bucketise(iterable: Iterable[_U], key: Callable[[_U], _T]) -> dict[_T, list[_U]]:
@@ -158,18 +185,18 @@ def run_in_thread(fn: Callable[..., object]) -> Callable[..., Awaitable[object]]
     return wrapper
 
 
-def tabulate(rows: Sequence[Sequence[object]], *, max_col_width: int = 60) -> str:
+def tabulate(rows: Sequence[tuple[object, ...]], *, max_col_width: int = 60) -> str:
     "Produce an ASCII table from equal-length elements in a sequence."
     from textwrap import fill
 
     def apply_max_col_width(value: object):
         return fill(str(value), width=max_col_width, max_lines=1)
 
-    def calc_resultant_col_widths(rows: Sequence[Sequence[str]]):
+    def calc_resultant_col_widths(rows: Sequence[tuple[str, ...]]):
         cols = zip(*rows)
         return [max(map(len, c)) for c in cols]
 
-    rows = [tuple(map(apply_max_col_width, r)) for r in rows]
+    rows = [tuple(apply_max_col_width(i) for i in r) for r in rows]
     head, *tail = rows
 
     base_template = '  '.join(f'{{{{{{0}}{w}}}}}' for w in calc_resultant_col_widths(rows))
@@ -177,7 +204,7 @@ def tabulate(rows: Sequence[Sequence[object]], *, max_col_width: int = 60) -> st
     table = '\n'.join(
         (
             base_template.format(':^').format(*head),
-            base_template.format(f'0:-<').format(''),
+            base_template.format('0:-<').format(''),
             *(row_template.format(*r) for r in tail),
         )
     )
@@ -191,15 +218,19 @@ def move(src: str | os.PathLike[str], dest: str | os.PathLike[str]) -> Any:
     )
 
 
-def trash(paths: Sequence[PurePath], *, dest: PurePath, missing_ok: bool = False) -> None:
+def trash(paths: Collection[PurePath], *, dest: PurePath, missing_ok: bool = False) -> None:
     if not paths:
         return
 
-    parent_folder = mkdtemp(dir=dest, prefix=f'deleted-{paths[0].name}-')
+    exc_classes = FileNotFoundError if missing_ok else ()
+
+    first_path_name = next(iter(paths)).name
+    parent_folder = mkdtemp(dir=dest, prefix=f'deleted-{first_path_name}-')
+
     for path in paths:
         try:
             move(path, dest=parent_folder)
-        except (FileNotFoundError if missing_ok else ()):
+        except exc_classes:
             pass
 
 
@@ -207,13 +238,13 @@ def shasum(*values: object) -> str:
     "Base-16-encode a string using SHA-256 truncated to 32 characters."
     from hashlib import sha256
 
-    return sha256(''.join(map(str, values)).encode()).hexdigest()[:32]
+    return sha256(''.join(map(str, filter(None, values))).encode()).hexdigest()[:32]
 
 
-def is_not_stale(path: Path, ttl: Mapping[str, float]) -> bool:
+def is_not_stale(path: Path, ttl: timedelta) -> bool:
     "Check if a file is older than ``ttl``."
     return path.exists() and (
-        (datetime.now() - datetime.fromtimestamp(path.stat().st_mtime)) < timedelta(**ttl)
+        (datetime.now() - datetime.fromtimestamp(path.stat().st_mtime)) < ttl
     )
 
 
@@ -226,7 +257,7 @@ def find_addon_zip_tocs(names: Iterable[str]) -> Iterator[tuple[str, str]]:
                 yield (name, head)
 
 
-def make_zip_member_filter(base_dirs: Set[str]) -> Callable[[str], bool]:
+def make_zip_member_filter_fn(base_dirs: Set[str]) -> Callable[[str], bool]:
     "Filter out items which are not sub-paths of top-level folders in a ZIP."
 
     def is_subpath(name: str):
@@ -236,6 +267,10 @@ def make_zip_member_filter(base_dirs: Set[str]) -> Callable[[str], bool]:
     return is_subpath
 
 
+def is_file_uri(uri: str) -> bool:
+    return uri.startswith('file://')
+
+
 def file_uri_to_path(file_uri: str) -> str:
     "Convert a file URI to a path that works both on Windows and *nix."
     from urllib.parse import unquote
@@ -271,3 +306,10 @@ def reveal_folder(path: str | os.PathLike[str]) -> None:
         import click
 
         click.launch(os.fspath(path), locate=True)
+
+
+@contextmanager
+def time_op(on_complete: Callable[[float], None]) -> Iterator[None]:
+    start = time.perf_counter()
+    yield
+    on_complete(time.perf_counter() - start)
