@@ -1,10 +1,10 @@
 from __future__ import annotations
 
 from collections.abc import Iterable, Iterator, Sequence
+from datetime import timedelta
 from functools import partial, reduce
 from itertools import chain, product
 import json
-import time
 import typing
 from typing import Any
 
@@ -14,8 +14,18 @@ from loguru import logger
 from typing_extensions import Literal, NotRequired as N, TypeAlias, TypedDict
 from yarl import URL
 
+from .http import CACHE_INDEFINITELY, make_generic_progress_ctx
 from .manager import Manager
-from .utils import StrEnum, bucketise, chain_dict, gather, run_in_thread as t, shasum
+from .utils import (
+    StrEnum,
+    bucketise,
+    chain_dict,
+    gather,
+    read_resource_as_text,
+    run_in_thread as t,
+    shasum,
+    time_op,
+)
 
 _Slug = str
 _ImportString = str
@@ -123,24 +133,26 @@ class _WagoApiResponse_Changelog(TypedDict):
 
 
 class _TocNumber(StrEnum):
-    retail = '90200'
+    retail = '90207'
     vanilla_classic = '11403'
-    burning_crusade_classic = '20504'
+    classic = '30400'
 
 
 class WaCompanionBuilder:
     """A WeakAuras Companion port for shellfolk."""
 
     def __init__(self, manager: Manager) -> None:
-        self.manager = manager
+        self._manager = manager
 
-        output_folder = self.manager.config.plugin_dir / __name__
+        output_folder = self._manager.config.plugin_dir / __name__
         self.addon_zip_path = output_folder / 'WeakAurasCompanion.zip'
         self.changelog_path = output_folder / 'CHANGELOG.md'
         self.version_txt_path = output_folder / 'version.txt'
 
-    def _get_access_token(self):
-        return self.manager.config.global_config.access_tokens.wago or ''
+    def _make_auth_headers(self):
+        access_token = self._manager.config.global_config.access_tokens.wago
+        if access_token:
+            return {'api-key': access_token}
 
     @staticmethod
     def extract_auras(model: type[WeakAuras | Plateroos], source: str) -> WeakAuras | Plateroos:
@@ -151,7 +163,7 @@ class WaCompanionBuilder:
         return model.from_lua_table(aura_table)
 
     def extract_installed_auras(self) -> Iterator[WeakAuras | Plateroos]:
-        flavour_root = self.manager.config.addon_dir.parents[1]
+        flavour_root = self._manager.config.addon_dir.parents[1]
         saved_vars_by_account = flavour_root.glob('WTF/Account/*/SavedVariables')
         for saved_vars, model in product(
             saved_vars_by_account,
@@ -162,17 +174,16 @@ class WaCompanionBuilder:
                 logger.info(f'{file} not found')
             else:
                 content = file.read_text(encoding='utf-8-sig', errors='replace')
-                aura_group_cache = self.manager.config.global_config.cache_dir / shasum(content)
+                aura_group_cache = self._manager.config.global_config.cache_dir / shasum(content)
                 if aura_group_cache.exists():
                     logger.info(f'loading {file} from cache at {aura_group_cache}')
                     aura_groups_json = json.loads(aura_group_cache.read_bytes())
                     aura_groups = _aura_converter.structure({'root': aura_groups_json}, model)
                 else:
-                    start = time.perf_counter()
-                    aura_groups = self.extract_auras(model, content)
-                    logger.debug(
-                        f'extracted {model.__name__} in {time.perf_counter() - start:.3f}s'
-                    )
+                    with time_op(
+                        lambda t: logger.debug(f'extracted {model.__name__} in {t:.3f}s')
+                    ):
+                        aura_groups = self.extract_auras(model, content)
                     aura_group_cache.write_text(
                         json.dumps(
                             _aura_converter.unstructure(  # pyright: ignore[reportUnknownMemberType]
@@ -184,27 +195,29 @@ class WaCompanionBuilder:
                 yield aura_groups
 
     async def _fetch_wago_metadata(self, api_ep: str, aura_ids: Iterable[str]):
-        async with self.manager.web_client.get(
+        async with self._manager.web_client.get(
             (_CHECK_API_URL / api_ep).with_query(ids=','.join(aura_ids)),
-            {'minutes': 30},
-            label='Fetching aura metadata',
-            headers={'api-key': self._get_access_token()},
+            expire_after=timedelta(minutes=30),
+            headers=self._make_auth_headers(),
+            trace_request_ctx=make_generic_progress_ctx('Fetching aura metadata'),
         ) as response:
             metadata: list[_WagoApiResponse]
             if response.status == 404:
                 metadata = []
                 return metadata
+
             response.raise_for_status()
+
             metadata = await response.json()
             return sorted(metadata, key=lambda r: r['slug'])
 
     async def _fetch_wago_import_string(self, aura: _WagoApiResponse):
-        async with self.manager.web_client.get(
+        async with self._manager.web_client.get(
             _IMPORT_STRING_API_URL.with_query(id=aura['_id']).with_fragment(str(aura['version'])),
-            {'days': 30},
-            headers={'api-key': self._get_access_token()},
-            label=f"Fetching aura '{aura['slug']}'",
+            expire_after=CACHE_INDEFINITELY,
+            headers=self._make_auth_headers(),
             raise_for_status=True,
+            trace_request_ctx=make_generic_progress_ctx(f"Fetching aura '{aura['slug']}'"),
         ) as response:
             return await response.text()
 
@@ -214,7 +227,10 @@ class WaCompanionBuilder:
 
         metadata = await self._fetch_wago_metadata(auras.api_ep, auras.root)
         import_strings = await gather(self._fetch_wago_import_string(r) for r in metadata)
-        return [(auras.root[r['slug']], r, i) for r, i in zip(metadata, import_strings)]
+        return [
+            (auras.root.get(r['slug']) or auras.root[r['_id']], r, i)
+            for r, i in zip(metadata, import_strings)
+        ]
 
     def _checksum(self):
         from hashlib import sha256
@@ -222,18 +238,18 @@ class WaCompanionBuilder:
         return sha256(self.addon_zip_path.read_bytes()).hexdigest()
 
     def _generate_addon(self, auras: Iterable[tuple[type, _AuraGroup]]):
-        from importlib.resources import read_text
         from zipfile import ZipFile, ZipInfo
 
         from jinja2 import Environment, FunctionLoader
 
-        from . import wa_templates
+        from . import _wa_templates
 
         jinja_env = Environment(
             trim_blocks=True,
             lstrip_blocks=True,
-            loader=FunctionLoader(partial(read_text, wa_templates)),
+            loader=FunctionLoader(partial(read_resource_as_text, _wa_templates)),
         )
+
         aura_dict = chain_dict((WeakAuras, Plateroos), (), auras)
 
         self.addon_zip_path.parent.mkdir(exist_ok=True)
@@ -307,7 +323,11 @@ class WaCompanionBuilder:
             write_tpl('init.lua', {})
             write_tpl(
                 'WeakAurasCompanion.toc',
-                {'interface': self.manager.config.game_flavour.to_flavour_keyed_enum(_TocNumber)},
+                {
+                    'interface': self._manager.config.game_flavour.to_flavour_keyed_enum(
+                        _TocNumber
+                    ).value
+                },
             )
 
         self.changelog_path.write_text(
