@@ -3,7 +3,7 @@ from datetime import datetime
 import click
 
 from instawow.common import ChangelogFormat, SourceMetadata, Strategy
-from instawow.models import Pkg
+from instawow.models import Pkg, PkgOptions
 import instawow.plugins
 from instawow.resolvers import BaseResolver, Defn
 
@@ -20,6 +20,7 @@ class MyResolver(BaseResolver):
         name="It's me",
         strategies=frozenset({Strategy.default}),
         changelog_format=ChangelogFormat.markdown,
+        addon_toc_key=None,
     )
 
     async def resolve_one(self, defn: Defn, metadata: None) -> Pkg:
@@ -34,7 +35,7 @@ class MyResolver(BaseResolver):
             date_published=datetime.now(),
             version='0',
             changelog_url='data:,',
-            options={'strategy': defn.strategy},
+            options=PkgOptions(strategy=defn.strategy),
         )
 
 
