@@ -16,10 +16,17 @@ import pytest
 
 from instawow import __version__
 from instawow.cli import cli
-from instawow.common import Flavour
 from instawow.config import Config
 
 
+@pytest.fixture(autouse=True, scope='module')
+def __mock_pt_progress_bar():
+    monkeypatch = pytest.MonkeyPatch()
+    monkeypatch.setattr('prompt_toolkit.shortcuts.progress_bar.ProgressBar', mock.MagicMock())
+    yield
+    monkeypatch.undo()
+
+
 @pytest.fixture
 def feed_pt():
     with create_pipe_input() as input_, create_app_session(input=input_, output=DummyOutput()):
@@ -32,9 +39,7 @@ def run(
     event_loop: asyncio.AbstractEventLoop,
     iw_config: Config,
 ):
-    monkeypatch.setattr('prompt_toolkit.shortcuts.progress_bar.ProgressBar', mock.MagicMock())
     monkeypatch.setattr('asyncio.run', event_loop.run_until_complete)
-    monkeypatch.setenv('INSTAWOW_CONFIG_DIR', str(iw_config.global_config.config_dir))
     yield partial(CliRunner().invoke, cli, catch_exceptions=False)
 
 
