@@ -6,7 +6,7 @@ from typing import Any
 
 import pytest
 
-from instawow.common import Flavour, infer_flavour_from_path
+from instawow.common import Flavour
 from instawow.config import Config, GlobalConfig
 
 
@@ -23,14 +23,14 @@ def test_env_vars_have_prio(
         **iw_config_values,
     )
     assert config.global_config.config_dir == Path('/foo').resolve()
-    assert config.game_flavour is Flavour.burning_crusade_classic
+    assert config.game_flavour is Flavour.classic
 
 
 def test_config_dir_is_populated(
     iw_global_config_values: dict[str, Any],
     iw_config_values: dict[str, Any],
 ):
-    global_config = GlobalConfig(**iw_global_config_values)
+    global_config = GlobalConfig.from_env(**iw_global_config_values)
     config = Config(global_config=global_config, **iw_config_values).write()
     assert {i.name for i in config.profile_dir.iterdir()} == {'config.json', 'logs', 'plugins'}
 
@@ -47,7 +47,7 @@ def test_init_with_nonexistent_addon_dir_raises(
     iw_global_config_values: dict[str, Any],
     iw_config_values: dict[str, Any],
 ):
-    global_config = GlobalConfig(**iw_global_config_values).write()
+    global_config = GlobalConfig.from_env(**iw_global_config_values).write()
     with pytest.raises(ValueError, match='not a writable directory'):
         Config(global_config=global_config, **{**iw_config_values, 'addon_dir': '#@$foo'})
 
@@ -86,26 +86,9 @@ def test_default_config_dir_is_unplatform_appropriate(
     assert GlobalConfig().config_dir == Path.home() / 'instawow'
 
 
-def test_can_infer_flavour_from_path():
-    # fmt: off
-    assert infer_flavour_from_path('wowzerz/_classic_/Interface/AddOns') is Flavour.burning_crusade_classic
-    assert infer_flavour_from_path('/foo/bar/_classic_beta_/Interface/AddOns') is Flavour.burning_crusade_classic
-    assert infer_flavour_from_path('/foo/bar/_classic_ptr_/Interface/AddOns') is Flavour.burning_crusade_classic
-    assert infer_flavour_from_path('_classic_era_/Interface/AddOns') is Flavour.vanilla_classic
-    assert infer_flavour_from_path('_classic_era_beta_/Interface/AddOns') is Flavour.vanilla_classic
-    assert infer_flavour_from_path('_classic_era_ptr_/Interface/AddOns') is Flavour.vanilla_classic
-    assert infer_flavour_from_path('wowzerz/_retail_/Interface/AddOns') is Flavour.retail
-    assert infer_flavour_from_path('anything goes') is Flavour.retail
-    # fmt: on
-
-
 def test_can_list_profiles(
-    monkeypatch: pytest.MonkeyPatch,
-    iw_global_config_values: dict[str, Any],
     iw_config_values: dict[str, Any],
 ):
-    monkeypatch.setenv('INSTAWOW_CONFIG_DIR', str(iw_global_config_values['config_dir']))
-
     global_config = GlobalConfig.read()
     assert global_config.list_profiles() == []
 
@@ -118,7 +101,7 @@ def test_can_delete_profile(
     iw_global_config_values: dict[str, Any],
     iw_config_values: dict[str, Any],
 ):
-    global_config = GlobalConfig(**iw_global_config_values).write()
+    global_config = GlobalConfig.from_env(**iw_global_config_values).write()
     config = Config(global_config=global_config, **iw_config_values).write()
     assert config.profile_dir.exists()
     config.delete()
