@@ -5,15 +5,34 @@ from pathlib import Path
 
 from aiohttp import ClientError
 from aresponses import ResponsesMockServer
+from attrs import evolve
 import pytest
 
 from instawow import results as R
 from instawow.common import Flavour, Strategy
+from instawow.config import GlobalConfig
 from instawow.manager import Manager, is_outdated
 from instawow.models import Pkg
 from instawow.resolvers import Defn
 
 
+def test_auth_bound_resolvers_are_not_unloaded_if_tokens_set(iw_manager: Manager):
+    assert {
+        r.metadata.id for r in iw_manager.RESOLVERS if r.requires_access_token is not None
+    }.issubset(iw_manager.resolvers)
+
+
+@pytest.mark.parametrize(
+    'iw_global_config_values',
+    [None],
+    indirect=True,
+)
+def test_auth_bound_resolvers_are_unloaded_if_tokens_unset(iw_manager: Manager):
+    assert {
+        r.metadata.id for r in iw_manager.RESOLVERS if r.requires_access_token is not None
+    }.isdisjoint(iw_manager.resolvers)
+
+
 async def test_pinning_supported_pkg(iw_manager: Manager):
     defn = Defn('curse', 'molinari')
     install_result = await iw_manager.install([defn], False)
@@ -54,7 +73,7 @@ async def test_resolve_rewraps_exception_appropriately_from_resolve(
     async def resolve_one(self, defn, metadata):
         raise exception
 
-    monkeypatch.setattr('instawow.resolvers.CfCoreResolver.resolve_one', resolve_one)
+    monkeypatch.setattr('instawow._sources.cfcore.CfCoreResolver.resolve_one', resolve_one)
 
     defn = Defn('curse', 'molinari')
     results = await iw_manager.resolve([defn])
@@ -69,7 +88,7 @@ async def test_resolve_rewraps_exception_appropriately_from_batch_resolve(
     async def resolve(self, defns):
         raise exception
 
-    monkeypatch.setattr('instawow.resolvers.CfCoreResolver.resolve', resolve)
+    monkeypatch.setattr('instawow._sources.cfcore.CfCoreResolver.resolve', resolve)
 
     defn = Defn('curse', 'molinari')
     results = await iw_manager.resolve([defn])
@@ -148,6 +167,23 @@ async def test_update_lifecycle_while_varying_retain_defn_strategy(iw_manager: M
     assert result[defn].new_pkg.options.strategy == Strategy.default
 
 
+async def test_update_reinstalls_corrupted_pkgs(iw_manager: Manager):
+    defn = Defn('curse', 'molinari')
+
+    result = await iw_manager.install([defn], replace=False)
+    assert type(result[defn]) is R.PkgInstalled
+
+    folders = [iw_manager.config.addon_dir / f.name for f in result[defn].pkg.folders]
+
+    first_folder = folders[0]
+    first_folder.rename(first_folder.with_name('foo'))
+    assert not all(f.is_dir() for f in folders)
+
+    result = await iw_manager.update([defn], retain_defn_strategy=False)
+    assert type(result[defn]) is R.PkgUpdated
+    assert all(f.is_dir() for f in folders)
+
+
 @pytest.mark.parametrize('keep_folders', [True, False])
 async def test_deleting_and_retaining_folders_on_remove(iw_manager: Manager, keep_folders: bool):
     defn = Defn('curse', 'molinari')
@@ -194,7 +230,7 @@ async def test_search_flavour_filtering(iw_manager: Manager):
     faux_defns = {(e.source, e.slug or e.id) for e in results}
     if iw_manager.config.game_flavour in {
         Flavour.vanilla_classic,
-        Flavour.burning_crusade_classic,
+        Flavour.classic,
     }:
         assert ('curse', 'atlaslootclassic') in faux_defns
     else:
@@ -245,23 +281,25 @@ async def test_get_changelog_from_web_url(iw_manager: Manager):
     ).startswith('<h3>Changes in 90200.82-Release:</h3>')
 
 
-@pytest.mark.iw_no_mock_http
+@pytest.mark.parametrize(
+    'iw_mock_aiohttp_requests',
+    [set()],
+    indirect=True,
+)
 async def test_is_outdated_works_in_variety_of_scenarios(
-    monkeypatch: pytest.MonkeyPatch, aresponses: ResponsesMockServer, iw_temp_dir: Path
+    monkeypatch: pytest.MonkeyPatch,
+    aresponses: ResponsesMockServer,
+    iw_global_config_values: dict[str, object],
 ):
-    pypi_version = iw_temp_dir.joinpath('.pypi_version')
-    if pypi_version.exists():
-        pypi_version.unlink()
+    global_config = GlobalConfig.from_env(**iw_global_config_values)
 
-    # version == '0.0.0', version not cached
+    # version == '0+dev', version not cached
     with monkeypatch.context() as patcher:
-        patcher.setattr('instawow.__version__', '0.0.0')
-        assert await is_outdated() == (False, '')
+        patcher.setattr('instawow.__version__', '0+dev')
+        assert await is_outdated(global_config) == (False, '')
 
     # Update check disabled, version not cached
-    with monkeypatch.context() as patcher:
-        patcher.setenv('INSTAWOW_AUTO_UPDATE_CHECK', '0')
-        assert await is_outdated() == (False, '')
+    assert await is_outdated(evolve(global_config, auto_update_check=False)) == (False, '')
 
     # Endpoint not responsive, version not cached
     with monkeypatch.context() as patcher:
@@ -272,7 +310,7 @@ async def test_is_outdated_works_in_variety_of_scenarios(
             'get',
             aresponses.Response(status=500),
         )
-        assert await is_outdated() == (False, '0.1.0')
+        assert await is_outdated(global_config) == (False, '0.1.0')
 
     # Endpoint responsive, version not cached and version different
     with monkeypatch.context() as patcher:
@@ -283,17 +321,15 @@ async def test_is_outdated_works_in_variety_of_scenarios(
             'get',
             {'info': {'version': '1.0.0'}},
         )
-        assert await is_outdated() == (True, '1.0.0')
+        assert await is_outdated(global_config) == (True, '1.0.0')
 
-    # version == '0.0.0', version cached
+    # version == '0+dev', version cached
     with monkeypatch.context() as patcher:
-        patcher.setattr('instawow.__version__', '0.0.0')
-        assert await is_outdated() == (False, '')
+        patcher.setattr('instawow.__version__', '0+dev')
+        assert await is_outdated(global_config) == (False, '')
 
     # Update check disabled, version cached
-    with monkeypatch.context() as patcher:
-        patcher.setenv('INSTAWOW_AUTO_UPDATE_CHECK', '0')
-        assert await is_outdated() == (False, '')
+    assert await is_outdated(evolve(global_config, auto_update_check=False)) == (False, '')
 
     # Endpoint not responsive, version cached
     with monkeypatch.context() as patcher:
@@ -304,7 +340,7 @@ async def test_is_outdated_works_in_variety_of_scenarios(
             'get',
             aresponses.Response(status=500),
         )
-        assert await is_outdated() == (True, '1.0.0')
+        assert await is_outdated(global_config) == (True, '1.0.0')
 
     # Endpoint responsive, version cached and version same
     with monkeypatch.context() as patcher:
@@ -315,7 +351,7 @@ async def test_is_outdated_works_in_variety_of_scenarios(
             'get',
             {'info': {'version': '1.0.0'}},
         )
-        assert await is_outdated() == (True, '1.0.0')
+        assert await is_outdated(global_config) == (True, '1.0.0')
 
     # Endpoint responsive, version cached and version different
     with monkeypatch.context() as patcher:
@@ -326,4 +362,4 @@ async def test_is_outdated_works_in_variety_of_scenarios(
             'get',
             {'info': {'version': '1.0.0'}},
         )
-        assert await is_outdated() == (False, '1.0.0')
+        assert await is_outdated(global_config) == (False, '1.0.0')
